@@ -40,7 +40,7 @@ class G:
     events = {}
     start = None
     markers = ["o", "s", "t", "d"]
-    curveno = 0
+    curveno = {}
 
 def find_marker():
     if len(G.markers) == 0:
@@ -66,6 +66,10 @@ def get_event(event, evtype):
         G.events[event] = e
     return G.events[event]
 
+colors = [(0.0, 0.5, 0.0), (0.0, 0.0, 1.0), (0.0, 0.0, 0.0), (1.0, 0.0, 0.0), (0.75, 0.75, 0), (0.0, 0.75, 0.75), (0.75, 0, 0.75)]
+def mkColor(t):
+    return pg.mkColor(int(t[0] * 255), int(t[1] * 255), int(t[2] * 255))
+
 SCALE = 1e6 # microseconds to seconds
 
 for line in [line.split("#")[0].strip() for line in open(filename, "r")]:
@@ -147,13 +151,16 @@ for cur in ax:
        cur.addLegend(offset = (-1, 1))
 for e in G.sevents:
     cur = ax[1 if e.type == "value" else 0]
+    if not cur in G.curveno:
+        G.curveno[cur] = 0
     args = {'name': e.name,'antialias':True}
+    color = mkColor(colors[G.curveno[cur] % len(colors)])
     if e.type == "event-signal":
         args['symbol'] = e.marker
-        args['pen'] = None
+        args['symbolBrush'] = pg.mkBrush(color, width=0)
     else:
-        args['pen'] = pg.mkPen(pg.intColor(G.curveno), width=0)
-        G.curveno += 1
+        args['pen'] = pg.mkPen(color, width=0)
+    G.curveno[cur] += 1
     n = cur.plot([x for x,y in e.vals], [y for x,y in e.vals], **args)
 
 QtGui.QApplication.instance().exec_()
