@@ -94,6 +94,7 @@ struct priv {
     snd_pcm_format_t alsa_fmt;
     bool can_pause;
     bool paused;
+    bool final_chunk_written;
     snd_pcm_sframes_t prepause_frames;
     double delay_before_pause;
     snd_pcm_uframes_t buffersize;
@@ -135,6 +136,19 @@ static bool check_device_present(struct ao *ao, int alsa_err)
     return false;
 }
 
+static void handle_underrun(struct ao *ao)
+{
+    struct priv *p = ao->priv;
+
+    if (!p->final_chunk_written) {
+        ao_underrun_event(ao);
+
+        int err = snd_pcm_prepare(p->alsa);
+        CHECK_ALSA_ERROR("pcm prepare error");
+    alsa_error: ;
+    }
+}
+
 static int control(struct ao *ao, enum aocontrol cmd, void *arg)
 {
     struct priv *p = ao->priv;
@@ -946,14 +960,7 @@ static int get_space(struct ao *ao)
         return p->buffersize;
 
     snd_pcm_sframes_t space = snd_pcm_avail(p->alsa);
-    if (space < 0) {
-        if (space == -EPIPE) {
-            MP_WARN(ao, "ALSA XRUN hit, attempting to recover...\n");
-            int err = snd_pcm_prepare(p->alsa);
-            CHECK_ALSA_ERROR("Unable to recover from under/overrun!");
-            return p->buffersize;
-        }
-
+    if (space < 0 && space != -EPIPE) {
         MP_ERR(ao, "Error received from snd_pcm_avail "
                    "(%ld, %s with ALSA state %s)!\n",
                space, snd_strerror(space),
@@ -964,8 +971,10 @@ static int get_space(struct ao *ao)
         check_device_present(ao, space);
         goto alsa_error;
     }
+    if (space == -EPIPE)
+        handle_underrun(ao);
 
-    if (space > p->buffersize) // Buffer underrun?
+    if (space > p->buffersize || space < 0) // Buffer underrun?
         space = p->buffersize;
     return space / p->outburst * p->outburst;
 
@@ -982,8 +991,12 @@ static double get_delay(struct ao *ao)
     if (p->paused)
         return p->delay_before_pause;
 
-    if (snd_pcm_delay(p->alsa, &delay) < 0)
+    int err = snd_pcm_delay(p->alsa, &delay);
+    if (err < 0) {
+        if (err == -EPIPE)
+            handle_underrun(ao);
         return 0;
+    }
 
     if (delay < 0) {
         /* underrun - move the application pointer forward to catch up */
@@ -1089,6 +1102,7 @@ static void reset(struct ao *ao)
     p->paused = false;
     p->prepause_frames = 0;
     p->delay_before_pause = 0;
+    p->final_chunk_written = false;
 
     if (ao->stream_silence) {
         soft_reset(ao);
@@ -1106,11 +1120,13 @@ static int play(struct ao *ao, void **data, int samples, int flags)
 {
     struct priv *p = ao->priv;
     snd_pcm_sframes_t res = 0;
-    if (!(flags & AOPLAY_FINAL_CHUNK))
+    bool final_chunk = flags & AOPLAY_FINAL_CHUNK;
+
+    if (!final_chunk)
         samples = samples / p->outburst * p->outburst;
 
     if (samples == 0)
-        return 0;
+        goto done;
     ao_convert_inplace(&p->convert, data, samples);
 
     do {
@@ -1128,15 +1144,11 @@ static int play(struct ao *ao, void **data, int samples, int flags)
             if (res == -ESTRPIPE) {  /* suspend */
                 resume_device(ao);
             } else if (res == -EPIPE) {
-                // For some reason, writing a smaller fragment at the end
-                // immediately underruns.
-                if (!(flags & AOPLAY_FINAL_CHUNK))
-                    MP_WARN(ao, "Device underrun detected.\n");
+                handle_underrun(ao);
             } else {
                 MP_ERR(ao, "Write error: %s\n", snd_strerror(res));
             }
-            res = snd_pcm_prepare(p->alsa);
-            int err = res;
+            int err = snd_pcm_prepare(p->alsa);
             CHECK_ALSA_ERROR("pcm prepare error");
             res = 0;
         }
@@ -1144,6 +1156,8 @@ static int play(struct ao *ao, void **data, int samples, int flags)
 
     p->paused = false;
 
+done:
+    p->final_chunk_written = res == samples && final_chunk;
     return res < 0 ? -1 : res;
 
 alsa_error:
@@ -1250,6 +1264,7 @@ const struct ao_driver audio_out_alsa = {
     .wait      = audio_wait,
     .wakeup    = ao_wakeup_poll,
     .list_devs = list_devs,
+    .reports_underruns = true,
     .priv_size = sizeof(struct priv),
     .global_opts = &ao_alsa_conf,
 };
