@@ -24,19 +24,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <inttypes.h>
-
-#ifdef __APPLE__
-#ifndef AL_FORMAT_MONO_FLOAT32
-#define AL_FORMAT_MONO_FLOAT32 0x10010
-#endif
-#ifndef AL_FORMAT_STEREO_FLOAT32
-#define AL_FORMAT_STEREO_FLOAT32 0x10011
-#endif
-#ifndef AL_FORMAT_MONO_DOUBLE_EXT
-#define AL_FORMAT_MONO_DOUBLE_EXT 0x10012
-#endif
-#include <OpenAL/MacOSX_OALExtensions.h>
-#else
 #ifdef OPENAL_AL_H
 #include <OpenAL/alc.h>
 #include <OpenAL/al.h>
@@ -46,7 +33,6 @@
 #include <AL/al.h>
 #include <AL/alext.h>
 #endif
-#endif // __APPLE__
 
 #include "common/msg.h"
 
@@ -107,8 +93,6 @@ static int control(struct ao *ao, enum aocontrol cmd, void *arg)
         return CONTROL_TRUE;
     }
 
-    case AOCONTROL_HAS_SOFT_VOLUME:
-        return CONTROL_TRUE;
     }
     return CONTROL_UNKNOWN;
 }
