@@ -49,10 +49,18 @@
 #include "ao.h"
 #include "internal.h"
 
+#if !HAVE_GPL
+#error GPL only
+#endif
+
 // Define to 0 if the device must be reopened to reset it (stop all playback,
 // clear the buffer), and the device should be closed when unused.
 // Define to 1 if SNDCTL_DSP_RESET should be used to reset without close.
-#define KEEP_DEVICE (defined(SNDCTL_DSP_RESET) && !defined(__NetBSD__))
+#if defined(SNDCTL_DSP_RESET) && !defined(__NetBSD__)
+#define KEEP_DEVICE 1
+#else
+#define KEEP_DEVICE 0
+#endif
 
 #define PATH_DEV_DSP "/dev/dsp"
 #define PATH_DEV_MIXER "/dev/mixer"
@@ -196,10 +204,6 @@ static int control(struct ao *ao, enum aocontrol cmd, void *arg)
         }
         return CONTROL_ERROR;
     }
-#ifdef SNDCTL_DSP_GETPLAYVOL
-    case AOCONTROL_HAS_SOFT_VOLUME:
-        return CONTROL_TRUE;
-#endif
     }
     return CONTROL_UNKNOWN;
 }
@@ -307,7 +311,7 @@ static int reopen_device(struct ao *ao, bool allow_format_changes)
         }
     }
 
-    int try_formats[AF_FORMAT_COUNT];
+    int try_formats[AF_FORMAT_COUNT + 1];
     af_get_best_sample_formats(format, try_formats);
     for (int n = 0; try_formats[n]; n++) {
         format = try_formats[n];
@@ -328,19 +332,23 @@ static int reopen_device(struct ao *ao, bool allow_format_changes)
             mp_chmap_sel_add_map(&sel, &oss_layouts[n]);
         if (!ao_chmap_sel_adjust(ao, &sel, &channels))
             goto fail;
-        int reqchannels = channels.num;
+        int c, nchannels, reqchannels;
+        nchannels = reqchannels = channels.num;
         // We only use SNDCTL_DSP_CHANNELS for >2 channels, in case some drivers don't have it
         if (reqchannels > 2) {
-            int nchannels = reqchannels;
-            if (ioctl(p->audio_fd, SNDCTL_DSP_CHANNELS, &nchannels) == -1 ||
-                nchannels != reqchannels)
-            {
+            if (ioctl(p->audio_fd, SNDCTL_DSP_CHANNELS, &nchannels) == -1) {
                 MP_ERR(ao, "Failed to set audio device to %d channels.\n",
                        reqchannels);
                 goto fail;
             }
+            if (nchannels != reqchannels) {
+                // Fallback to stereo
+                nchannels = 2;
+                goto stereo;
+            }
         } else {
-            int c = reqchannels - 1;
+stereo:
+            c = nchannels - 1;
             if (ioctl(p->audio_fd, SNDCTL_DSP_STEREO, &c) == -1) {
                 MP_ERR(ao, "Failed to set audio device to %d channels.\n",
                        reqchannels);
@@ -641,7 +649,7 @@ const struct ao_driver audio_out_oss = {
         .oss_mixer_device = PATH_DEV_MIXER,
     },
     .options = (const struct m_option[]) {
-        OPT_STRING("mixer-device", oss_mixer_device, 0),
+        OPT_STRING("mixer-device", oss_mixer_device, M_OPT_FILE),
         OPT_STRING("mixer-channel", cfg_oss_mixer_channel, 0),
         {0}
     },
