@@ -61,6 +61,7 @@ struct priv {
     char *cfg_host;
     int cfg_buffer;
     int cfg_latency_hacks;
+    int cfg_allow_suspended;
 };
 
 #define GENERIC_ERR_MSG(str) \
@@ -201,13 +202,18 @@ static const struct format_map {
 static pa_encoding_t map_digital_format(int format)
 {
     switch (format) {
-    case AF_FORMAT_S_AC3:   return PA_ENCODING_AC3_IEC61937;
-    case AF_FORMAT_S_EAC3:  return PA_ENCODING_EAC3_IEC61937;
-    case AF_FORMAT_S_MP3:   return PA_ENCODING_MPEG_IEC61937;
-    case AF_FORMAT_S_DTS:
-    case AF_FORMAT_S_DTSHD: return PA_ENCODING_DTS_IEC61937;
+    case AF_FORMAT_S_AC3:    return PA_ENCODING_AC3_IEC61937;
+    case AF_FORMAT_S_EAC3:   return PA_ENCODING_EAC3_IEC61937;
+    case AF_FORMAT_S_MP3:    return PA_ENCODING_MPEG_IEC61937;
+    case AF_FORMAT_S_DTS:    return PA_ENCODING_DTS_IEC61937;
+#ifdef PA_ENCODING_DTSHD_IEC61937
+    case AF_FORMAT_S_DTSHD:  return PA_ENCODING_DTSHD_IEC61937;
+#endif
 #ifdef PA_ENCODING_MPEG2_AAC_IEC61937
-    case AF_FORMAT_S_AAC:   return PA_ENCODING_MPEG2_AAC_IEC61937;
+    case AF_FORMAT_S_AAC:    return PA_ENCODING_MPEG2_AAC_IEC61937;
+#endif
+#ifdef PA_ENCODING_TRUEHD_IEC61937
+    case AF_FORMAT_S_TRUEHD: return PA_ENCODING_TRUEHD_IEC61937;
 #endif
     default:
         if (af_fmt_is_spdif(format))
@@ -482,7 +488,7 @@ static int init(struct ao *ao)
         pa_threaded_mainloop_wait(priv->mainloop);
     }
 
-    if (pa_stream_is_suspended(priv->stream)) {
+    if (pa_stream_is_suspended(priv->stream) && !priv->cfg_allow_suspended) {
         MP_ERR(ao, "The stream is suspended. Bailing out.\n");
         goto unlock_and_fail;
     }
@@ -743,9 +749,6 @@ static int control(struct ao *ao, enum aocontrol cmd, void *arg)
         return CONTROL_OK;
     }
 
-    case AOCONTROL_HAS_PER_APP_VOLUME:
-        return CONTROL_TRUE;
-
     case AOCONTROL_UPDATE_STREAM_TITLE: {
         char *title = (char *)arg;
         pa_threaded_mainloop_lock(priv->mainloop);
@@ -837,6 +840,7 @@ const struct ao_driver audio_out_pulse = {
         OPT_STRING("host", cfg_host, 0),
         OPT_CHOICE_OR_INT("buffer", cfg_buffer, 0, 1, 2000, ({"native", 0})),
         OPT_FLAG("latency-hacks", cfg_latency_hacks, 0),
+        OPT_FLAG("allow-suspended", cfg_allow_suspended, 0),
         {0}
     },
     .options_prefix = "pulse",
