@@ -153,8 +153,13 @@ int ao_read_data(struct ao *ao, void **data, int samples, int64_t out_time_us)
     int buffered_bytes = mp_ring_buffered(p->buffers[0]);
     bytes = MPMIN(buffered_bytes, full_bytes);
 
-    if (buffered_bytes < bytes && !atomic_load(&p->draining))
-        atomic_fetch_add(&p->underflow, (bytes - buffered_bytes) / ao->sstride);
+    if (full_bytes > bytes && !atomic_load(&p->draining)) {
+        if (ao->driver->reports_underruns) {
+            ao_underrun_event(ao);
+        } else {
+            atomic_fetch_add(&p->underflow, (full_bytes - bytes) / ao->sstride);
+        }
+    }
 
     if (bytes > 0)
         atomic_store(&p->end_time_us, out_time_us);
@@ -179,6 +184,8 @@ end:
     for (int n = 0; n < ao->num_planes; n++)
         af_fill_silence((char *)data[n] + bytes, full_bytes - bytes, ao->format);
 
+    ao_post_process_data(ao, data, samples);
+
     return bytes / ao->sstride;
 }
 
@@ -190,7 +197,7 @@ int ao_read_data_converted(struct ao *ao, struct ao_convert_fmt *fmt,
     assert(ao->api == &ao_api_pull);
 
     struct ao_pull_state *p = ao->api_priv;
-    void *ndata[MP_NUM_CHANNELS];
+    void *ndata[MP_NUM_CHANNELS] = {0};
 
     if (!ao_need_conversion(fmt))
         return ao_read_data(ao, data, samples, out_time_us);
