@@ -16,8 +16,7 @@ export RANLIB=$TARGET-ranlib
 export CFLAGS="-O2 -mtune=intel -g -ggdb -pipe -Wall --param=ssp-buffer-size=4 -mms-bitfields -fmessage-length=0 -D_FORTIFY_SOURCE=2 -fexceptions -fasynchronous-unwind-tables -fstack-protector-strong -fno-ident"
 export LDFLAGS="-Wl,--no-keep-memory -fstack-protector-strong"
 
-./bootstrap.py
-./waf configure \
+python3 ./waf configure \
     --enable-static-build \
     --enable-libmpv-shared \
     --enable-lua \
@@ -25,11 +24,10 @@ export LDFLAGS="-Wl,--no-keep-memory -fstack-protector-strong"
     --enable-libarchive \
     --enable-libass \
     --enable-libbluray \
-    --enable-dvdread \
     --enable-dvdnav \
     --enable-uchardet \
     --enable-vulkan \
     --enable-shaderc \
     --enable-rubberband \
     --enable-lcms2
-./waf build --verbose
+python3 ./waf build --verbose
