@@ -5,12 +5,10 @@ python3 ./waf configure \
   --enable-cdda          \
   --enable-dvbin         \
   --enable-dvdnav        \
-  --enable-dvdread       \
   --enable-libarchive    \
   --enable-libmpv-shared \
   --enable-libsmbclient  \
   --enable-manpage-build \
   --enable-shaderc       \
-  --enable-vulkan        \
-  --enable-zsh-comp
+  --enable-vulkan
 python3 ./waf build --verbose
