@@ -26,6 +26,7 @@
 #include <libavutil/error.h>
 #include <libavutil/cpu.h>
 #include <libavcodec/avcodec.h>
+#include <libavformat/avformat.h>
 
 #include "config.h"
 
@@ -33,6 +34,7 @@
 #include "common/msg.h"
 #include "demux/packet.h"
 #include "demux/stheader.h"
+#include "misc/bstr.h"
 #include "video/fmt-conversion.h"
 #include "av_common.h"
 #include "codecs.h"
@@ -217,38 +219,47 @@ void mp_set_avcodec_threads(struct mp_log *l, AVCodecContext *avctx, int threads
     avctx->thread_count = threads;
 }
 
-static bool is_crap(AVCodec *codec)
+static void add_codecs(struct mp_decoder_list *list, enum AVMediaType type,
+                       bool decoders)
 {
-    return !!strstr(codec->name, "_vdpau");
+    void *iter = NULL;
+    for (;;) {
+        const AVCodec *cur = av_codec_iterate(&iter);
+        if (!cur)
+            break;
+        if (av_codec_is_decoder(cur) == decoders &&
+            (type == AVMEDIA_TYPE_UNKNOWN || cur->type == type))
+        {
+            mp_add_decoder(list, mp_codec_from_av_codec_id(cur->id),
+                           cur->name, cur->long_name);
+        }
+    }
 }
 
 void mp_add_lavc_decoders(struct mp_decoder_list *list, enum AVMediaType type)
 {
-    AVCodec *cur = NULL;
-    for (;;) {
-        cur = av_codec_next(cur);
-        if (!cur)
-            break;
-        if (av_codec_is_decoder(cur) && cur->type == type && !is_crap(cur)) {
-            mp_add_decoder(list, "lavc", mp_codec_from_av_codec_id(cur->id),
-                           cur->name, cur->long_name);
-        }
-    }
+    add_codecs(list, type, true);
 }
 
 // (Abuses the decoder list data structures.)
 void mp_add_lavc_encoders(struct mp_decoder_list *list)
 {
-    AVCodec *cur = NULL;
+    add_codecs(list, AVMEDIA_TYPE_UNKNOWN, false);
+}
+
+char **mp_get_lavf_demuxers(void)
+{
+    char **list = NULL;
+    void *iter = NULL;
+    int num = 0;
     for (;;) {
-        cur = av_codec_next(cur);
+        const AVInputFormat *cur = av_demuxer_iterate(&iter);
         if (!cur)
             break;
-        if (av_codec_is_encoder(cur)) {
-            mp_add_decoder(list, "lavc", mp_codec_from_av_codec_id(cur->id),
-                           cur->name, cur->long_name);
-        }
+        MP_TARRAY_APPEND(NULL, list, num, talloc_strdup(NULL, cur->name));
     }
+    MP_TARRAY_APPEND(NULL, list, num, NULL);
+    return list;
 }
 
 int mp_codec_to_av_codec_id(const char *codec)
@@ -281,6 +292,13 @@ const char *mp_codec_from_av_codec_id(int codec_id)
     return name;
 }
 
+bool mp_codec_is_lossless(const char *codec)
+{
+    const AVCodecDescriptor *desc =
+        avcodec_descriptor_get(mp_codec_to_av_codec_id(codec));
+    return desc && (desc->props & AV_CODEC_PROP_LOSSLESS);
+}
+
 // kv is in the format as by OPT_KEYVALUELIST(): kv[0]=key0, kv[1]=val0, ...
 // Copy them to the dict.
 void mp_set_avdict(AVDictionary **dict, char **kv)
@@ -302,7 +320,7 @@ void mp_avdict_print_unset(struct mp_log *log, int msgl, AVDictionary *dict)
 // to the name of the n-th parameter.
 static void resolve_positional_arg(void *avobj, char **name)
 {
-    if (!*name || (*name)[0] != '@')
+    if (!*name || (*name)[0] != '@' || !avobj)
         return;
 
     char *end = NULL;
@@ -333,12 +351,19 @@ static void resolve_positional_arg(void *avobj, char **name)
 // Options which fail to set (error or not found) are printed to log.
 // Returns: >=0 success, <0 failed to set an option
 int mp_set_avopts(struct mp_log *log, void *avobj, char **kv)
+{
+    return mp_set_avopts_pos(log, avobj, avobj, kv);
+}
+
+// Like mp_set_avopts(), but the posargs argument is used to resolve positional
+// arguments. If posargs==NULL, positional args are disabled.
+int mp_set_avopts_pos(struct mp_log *log, void *avobj, void *posargs, char **kv)
 {
     int success = 0;
     for (int n = 0; kv && kv[n * 2]; n++) {
         char *k = kv[n * 2 + 0];
         char *v = kv[n * 2 + 1];
-        resolve_positional_arg(avobj, &k);
+        resolve_positional_arg(posargs, &k);
         int r = av_opt_set(avobj, k, v, AV_OPT_SEARCH_CHILDREN);
         if (r == AVERROR_OPTION_NOT_FOUND) {
             mp_err(log, "AVOption '%s' not found.\n", k);
