@@ -20,12 +20,17 @@
 
 #include "common/common.h"
 #include "common/playlist.h"
+#include "options/m_config.h"
 #include "stream/stream.h"
 #include "misc/natural_sort.h"
 #include "demux.h"
 
 #include "stream/stream_libarchive.h"
 
+struct demux_libarchive_opts {
+    int rar_list_all_volumes;
+};
+
 static int cmp_filename(const void *a, const void *b)
 {
     return mp_natural_sort_cmp(*(char **)a, *(char **)b);
@@ -49,7 +54,7 @@ static int open_file(struct demuxer *demuxer, enum demux_check check)
     int probe_got = stream_read_peek(demuxer->stream, probe, probe_size);
     struct stream *probe_stream =
         stream_memory_open(demuxer->global, probe, probe_got);
-    struct mp_archive *mpa = mp_archive_new(mp_null_log, probe_stream, flags);
+    struct mp_archive *mpa = mp_archive_new(mp_null_log, probe_stream, flags, 0);
     bool ok = !!mpa;
     free_stream(probe_stream);
     mp_archive_free(mpa);
@@ -57,7 +62,13 @@ static int open_file(struct demuxer *demuxer, enum demux_check check)
     if (!ok)
         return -1;
 
-    mpa = mp_archive_new(demuxer->log, demuxer->stream, flags);
+    struct demux_libarchive_opts *opts =
+        mp_get_config_group(demuxer, demuxer->global, demuxer->desc->options);
+
+    if (!opts->rar_list_all_volumes)
+        flags |= MP_ARCHIVE_FLAG_NO_RAR_VOLUMES;
+
+    mpa = mp_archive_new(demuxer->log, demuxer->stream, flags, 0);
     if (!mpa)
         return -1;
 
@@ -82,8 +93,7 @@ static int open_file(struct demuxer *demuxer, enum demux_check check)
     for (int n = 0; n < num_files; n++)
         playlist_add_file(pl, files[n]);
 
-    for (struct playlist_entry *e = pl->first; e; e = e->next)
-        e->stream_flags = demuxer->stream_origin;
+    playlist_set_stream_flags(pl, demuxer->stream_origin);
 
     demuxer->filetype = "archive";
     demuxer->fully_read = true;
@@ -94,8 +104,17 @@ static int open_file(struct demuxer *demuxer, enum demux_check check)
     return 0;
 }
 
+#define OPT_BASE_STRUCT struct demux_libarchive_opts
+
 const struct demuxer_desc demuxer_desc_libarchive = {
     .name = "libarchive",
     .desc = "libarchive wrapper",
     .open = open_file,
+    .options = &(const struct m_sub_options){
+        .opts = (const struct m_option[]) {
+            OPT_FLAG("rar-list-all-volumes", rar_list_all_volumes, 0),
+            {0}
+        },
+        .size = sizeof(OPT_BASE_STRUCT),
+    },
 };
