@@ -22,6 +22,7 @@
 #include <assert.h>
 
 #include <libavutil/buffer.h>
+#include <libavutil/common.h>
 #include <libavutil/rational.h>
 
 #include "config.h"
@@ -161,7 +162,7 @@ static void destroy(struct mp_filter *f)
 {
     struct priv *p = f->priv;
     if (p->decoder) {
-        MP_VERBOSE(f, "Uninit decoder.\n");
+        MP_DBG(f, "Uninit decoder.\n");
         talloc_free(p->decoder->f);
         p->decoder = NULL;
     }
@@ -660,11 +661,13 @@ static void read_frame(struct priv *p)
 
     if (p->decoded_coverart.type) {
         if (p->coverart_returned == 0) {
-            mp_pin_in_write(pin, mp_frame_ref(p->decoded_coverart));
+            frame = mp_frame_ref(p->decoded_coverart);
             p->coverart_returned = 1;
+            goto output_frame;
         } else if (p->coverart_returned == 1) {
-            mp_pin_in_write(pin, MP_EOF_FRAME);
+            frame = MP_EOF_FRAME;
             p->coverart_returned = 2;
+            goto output_frame;
         }
         return;
     }
@@ -772,7 +775,7 @@ struct mp_decoder_wrapper *mp_decoder_wrapper_create(struct mp_filter *parent,
 
     struct priv *p = f->priv;
     struct mp_decoder_wrapper *w = &p->public;
-    p->opt_cache = m_config_cache_alloc(p, f->global, GLOBAL_CONFIG);
+    p->opt_cache = m_config_cache_alloc(p, f->global, &mp_opt_root);
     p->log = f->log;
     p->f = f;
     p->header = src;
@@ -814,22 +817,23 @@ error:
     return NULL;
 }
 
-void lavc_process(struct mp_filter *f, bool *eof_flag,
-                  bool (*send)(struct mp_filter *f, struct demux_packet *pkt),
-                  bool (*receive)(struct mp_filter *f, struct mp_frame *res))
+void lavc_process(struct mp_filter *f, struct lavc_state *state,
+                  int (*send)(struct mp_filter *f, struct demux_packet *pkt),
+                  int (*receive)(struct mp_filter *f, struct mp_frame *res))
 {
     if (!mp_pin_in_needs_data(f->ppins[1]))
         return;
 
     struct mp_frame frame = {0};
-    if (!receive(f, &frame)) {
-        if (!*eof_flag)
-            mp_pin_in_write(f->ppins[1], MP_EOF_FRAME);
-        *eof_flag = true;
-    } else if (frame.type) {
-        *eof_flag = false;
+    int ret_recv = receive(f, &frame);
+    if (frame.type) {
+        state->eof_returned = false;
         mp_pin_in_write(f->ppins[1], frame);
-    } else {
+    } else if (ret_recv == AVERROR_EOF) {
+        if (!state->eof_returned)
+            mp_pin_in_write(f->ppins[1], MP_EOF_FRAME);
+        state->eof_returned = true;
+    } else if (ret_recv == AVERROR(EAGAIN)) {
         // Need to feed a packet.
         frame = mp_pin_out_read(f->ppins[0]);
         struct demux_packet *pkt = NULL;
@@ -843,7 +847,8 @@ void lavc_process(struct mp_filter *f, bool *eof_flag,
             }
             return;
         }
-        if (!send(f, pkt)) {
+        int ret_send = send(f, pkt);
+        if (ret_send == AVERROR(EAGAIN)) {
             // Should never happen, but can happen with broken decoders.
             MP_WARN(f, "could not consume packet\n");
             mp_pin_out_unread(f->ppins[0], frame);
@@ -852,5 +857,8 @@ void lavc_process(struct mp_filter *f, bool *eof_flag,
         }
         talloc_free(pkt);
         mp_filter_internal_mark_progress(f);
+    } else {
+        // Decoding error, or hwdec fallback recovery. Just try again.
+        mp_filter_internal_mark_progress(f);
     }
 }
