@@ -23,7 +23,8 @@ struct mp_sws_filter *mp_sws_filter_create(struct mp_filter *parent);
 // formats. This tries to set the output format to the one that will result in
 // the least loss. Returns a format from out_formats[], or 0 if no format could
 // be chosen (or it's not supported by libswscale).
-int mp_sws_find_best_out_format(int in_format, int *out_formats,
+int mp_sws_find_best_out_format(struct mp_sws_filter *sws,
+                                int in_format, int *out_formats,
                                 int num_out_formats);
 
 // Whether the given format is supported as input format.
