@@ -36,6 +36,32 @@
 #include "common/msg.h"
 #include "common/common.h"
 
+static int m_property_multiply(struct mp_log *log,
+                               const struct m_property *prop_list,
+                               const char *property, double f, void *ctx)
+{
+    union m_option_value val = {0};
+    struct m_option opt = {0};
+    int r;
+
+    r = m_property_do(log, prop_list, property, M_PROPERTY_GET_CONSTRICTED_TYPE,
+                      &opt, ctx);
+    if (r != M_PROPERTY_OK)
+        return r;
+    assert(opt.type);
+
+    if (!opt.type->multiply)
+        return M_PROPERTY_NOT_IMPLEMENTED;
+
+    r = m_property_do(log, prop_list, property, M_PROPERTY_GET, &val, ctx);
+    if (r != M_PROPERTY_OK)
+        return r;
+    opt.type->multiply(&opt, &val, f);
+    r = m_property_do(log, prop_list, property, M_PROPERTY_SET, &val, ctx);
+    m_option_free(&opt, &val);
+    return r;
+}
+
 struct m_property *m_property_list_find(const struct m_property *list,
                                         const char *name)
 {
@@ -107,6 +133,9 @@ int m_property_do(struct mp_log *log, const struct m_property *prop_list,
         struct mpv_node node = { .format = MPV_FORMAT_STRING, .u.string = arg };
         return m_property_do(log, prop_list, name, M_PROPERTY_SET_NODE, &node, ctx);
     }
+    case M_PROPERTY_MULTIPLY: {
+        return m_property_multiply(log, prop_list, name, *(double *)arg, ctx);
+    }
     case M_PROPERTY_SWITCH: {
         if (!log)
             return M_PROPERTY_ERROR;
