@@ -27,6 +27,6 @@ struct mpv_global;
 int m_config_parse_mp_command_line(m_config_t *config, struct playlist *files,
                                    struct mpv_global *global, char **argv);
 void m_config_preparse_command_line(m_config_t *config, struct mpv_global *global,
-                                    char **argv);
+                                    int *verbose, char **argv);
 
 #endif /* MPLAYER_PARSER_MPCMD_H */
