@@ -18,12 +18,11 @@
  */
 
 #import <Cocoa/Cocoa.h>
-#import "ar/HIDRemote.h"
 #include "osdep/macosx_events.h"
 
 struct input_ctx;
 
-@interface EventsResponder : NSObject <HIDRemoteDelegate>
+@interface EventsResponder : NSObject
 
 + (EventsResponder *)sharedInstance;
 - (void)setInputContext:(struct input_ctx *)ctx;
@@ -33,10 +32,11 @@ struct input_ctx;
 - (void)waitForInputContext;
 - (void)wakeup;
 - (void)putKey:(int)keycode;
-- (void)setHighestPriotityMediaKeysTap;
 - (void)handleFilesArray:(NSArray *)files;
 
 - (bool)queueCommand:(char *)cmd;
 - (bool)processKeyEvent:(NSEvent *)event;
 
+- (BOOL)handleMPKey:(int)key withMask:(int)mask;
+
 @end
