@@ -144,6 +144,12 @@
                         @"key"        : @"O",
                         @"target"     : self
                     }],
+                    [NSMutableDictionary dictionaryWithDictionary:@{
+                        @"name"       : @"Open Playlist…",
+                        @"action"     : @"openPlaylist",
+                        @"key"        : @"",
+                        @"target"     : self
+                    }],
                     @{ @"name": @"separator" },
                     [NSMutableDictionary dictionaryWithDictionary:@{
                         @"name"       : @"Close",
@@ -587,7 +593,7 @@
                         @"action"     : @"url:",
                         @"key"        : @"",
                         @"target"     : self,
-                        @"url"        : @"https://github.com/mpv-player/mpv/issues/new"
+                        @"url"        : @"https://github.com/mpv-player/mpv/issues/new/choose"
                     }]
                 ]
             }
@@ -703,6 +709,17 @@
     }
 }
 
+- (void)openPlaylist
+{
+    NSOpenPanel *panel = [[NSOpenPanel alloc] init];
+
+    if ([panel runModal] == NSModalResponseOK){
+        NSString *pl = [NSString stringWithFormat:@"loadlist \"%@\"",
+                                                  [panel URLs][0].path];
+        [(Application *)NSApp queueCommand:(char *)[pl UTF8String]];
+    }
+}
+
 - (void)openURL
 {
     NSAlert *alert = [[NSAlert alloc] init];
