@@ -27,6 +27,7 @@
 #include "common/common.h"
 #include "stream/stream.h"
 #include "misc/bstr.h"
+#include "misc/thread_tools.h"
 
 static void write_arg(bstr *cmdline, char *arg)
 {
@@ -251,7 +252,7 @@ int mp_subprocess(char **args, struct mp_cancel *cancel, void *ctx,
     STARTUPINFOEXW si = {
         .StartupInfo = {
             .cb = sizeof(si),
-            .dwFlags = STARTF_USESTDHANDLES,
+            .dwFlags = STARTF_USESTDHANDLES | STARTF_FORCEOFFFEEDBACK,
             .hStdInput = NULL,
             .hStdOutput = pipes[0].write,
             .hStdError = pipes[1].write,
