@@ -15,12 +15,14 @@
  * License along with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <errno.h>
 #include <stddef.h>
 #include <stdbool.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <utime.h>
 
 #include <libavutil/md5.h>
 
@@ -94,8 +96,7 @@ static int try_load_config(struct MPContext *mpctx, const char *file, int flags,
 
 // Set options file-local, and don't set them if the user set them via the
 // command line.
-#define FILE_LOCAL_FLAGS \
-    (M_SETOPT_BACKUP | M_SETOPT_RUNTIME | M_SETOPT_PRESERVE_CMDLINE)
+#define FILE_LOCAL_FLAGS (M_SETOPT_BACKUP | M_SETOPT_PRESERVE_CMDLINE)
 
 static void mp_load_per_file_config(struct MPContext *mpctx)
 {
@@ -156,6 +157,32 @@ void mp_load_auto_profiles(struct MPContext *mpctx)
 
 #define MP_WATCH_LATER_CONF "watch_later"
 
+static bool check_mtime(const char *f1, const char *f2)
+{
+    struct stat st1, st2;
+    if (stat(f1, &st1) != 0 || stat(f2, &st2) != 0)
+        return false;
+    return st1.st_mtime == st2.st_mtime;
+}
+
+static bool copy_mtime(const char *f1, const char *f2)
+{
+    struct stat st1, st2;
+
+    if (stat(f1, &st1) != 0 || stat(f2, &st2) != 0)
+        return false;
+
+    struct utimbuf ut = {
+        .actime = st2.st_atime,  // we want to pass this through intact
+        .modtime = st1.st_mtime,
+    };
+
+    if (utime(f2, &ut) != 0)
+        return false;
+
+    return true;
+}
+
 static char *mp_get_playback_resume_config_filename(struct MPContext *mpctx,
                                                     const char *fname)
 {
@@ -291,6 +318,11 @@ static void write_redirect(struct MPContext *mpctx, char *path)
             write_filename(mpctx, file, path);
             fclose(file);
         }
+
+        if (mpctx->opts->position_check_mtime &&
+            !mp_is_url(bstr0(path)) && !copy_mtime(path, conffile))
+            MP_WARN(mpctx, "Can't copy mtime from %s to %s\n", path, conffile);
+
         talloc_free(conffile);
     }
 }
@@ -303,10 +335,6 @@ void mp_write_watch_later_conf(struct MPContext *mpctx)
         goto exit;
 
     struct demuxer *demux = mpctx->demuxer;
-    if (demux && (!demux->seekable || demux->partially_seekable)) {
-        MP_INFO(mpctx, "Not seekable - not saving state.\n");
-        goto exit;
-    }
 
     conffile = mp_get_playback_resume_config_filename(mpctx, cur->filename);
     if (!conffile)
@@ -323,8 +351,14 @@ void mp_write_watch_later_conf(struct MPContext *mpctx)
     write_filename(mpctx, file, cur->filename);
 
     double pos = get_current_time(mpctx);
-    if (pos != MP_NOPTS_VALUE)
+
+    if ((demux && (!demux->seekable || demux->partially_seekable)) ||
+        pos == MP_NOPTS_VALUE)
+    {
+        MP_INFO(mpctx, "Not seekable, or time unknown - not saving position.\n");
+    } else {
         fprintf(file, "start=%f\n", pos);
+    }
     for (int i = 0; backup_properties[i]; i++) {
         const char *pname = backup_properties[i];
         char *val = NULL;
@@ -347,6 +381,14 @@ void mp_write_watch_later_conf(struct MPContext *mpctx)
     }
     fclose(file);
 
+    if (mpctx->opts->position_check_mtime &&
+        !mp_is_url(bstr0(cur->filename)) &&
+        !copy_mtime(cur->filename, conffile))
+    {
+        MP_WARN(mpctx, "Can't copy mtime from %s to %s\n", cur->filename,
+                conffile);
+    }
+
     // This allows us to recursively resume directories etc., whose entries are
     // expanded the first time it's "played". For example, if "/a/b/c.mkv" is
     // the current entry, then we want to resume this file if the user does
@@ -388,6 +430,13 @@ void mp_load_playback_resume(struct MPContext *mpctx, const char *file)
         return;
     char *fname = mp_get_playback_resume_config_filename(mpctx, file);
     if (fname && mp_path_exists(fname)) {
+        if (mpctx->opts->position_check_mtime &&
+            !mp_is_url(bstr0(file)) && !check_mtime(file, fname))
+        {
+            talloc_free(fname);
+            return;
+        }
+
         // Never apply the saved start position to following files
         m_config_backup_opt(mpctx->mconfig, "start");
         MP_INFO(mpctx, "Resuming playback. This behavior can "
@@ -408,7 +457,8 @@ struct playlist_entry *mp_check_playlist_resume(struct MPContext *mpctx,
 {
     if (!mpctx->opts->position_resume)
         return NULL;
-    for (struct playlist_entry *e = playlist->first; e; e = e->next) {
+    for (int n = 0; n < playlist->num_entries; n++) {
+        struct playlist_entry *e = playlist->entries[n];
         char *conf = mp_get_playback_resume_config_filename(mpctx, e->filename);
         bool exists = conf && mp_path_exists(conf);
         talloc_free(conf);
