@@ -92,6 +92,7 @@ static void vo_chain_reset_state(struct vo_chain *vo_c)
 {
     vo_seek_reset(vo_c->vo);
     vo_c->underrun = false;
+    vo_c->underrun_signaled = false;
 }
 
 void reset_video_state(struct MPContext *mpctx)
@@ -384,10 +385,9 @@ static void handle_new_frame(struct MPContext *mpctx)
         }
     }
     mpctx->delay -= frame_time;
-    if (mpctx->video_status >= STATUS_PLAYING) {
-        mpctx->time_frame += frame_time / mpctx->video_speed;
+    mpctx->time_frame += frame_time / mpctx->video_speed;
+    if (mpctx->video_status >= STATUS_PLAYING)
         adjust_sync(mpctx, pts, frame_time);
-    }
     MP_TRACE(mpctx, "frametime=%5.3f\n", frame_time);
 }
 
@@ -453,10 +453,22 @@ static bool have_new_frame(struct MPContext *mpctx, bool eof)
 
 // Fill mpctx->next_frames[] with a newly filtered or decoded image.
 // returns VD_* code
-static int video_output_image(struct MPContext *mpctx)
+static int video_output_image(struct MPContext *mpctx, bool *logical_eof)
 {
     struct vo_chain *vo_c = mpctx->vo_chain;
-    bool hrseek = mpctx->hrseek_active && mpctx->video_status == STATUS_SYNCING;
+    bool hrseek = false;
+    double hrseek_pts = mpctx->hrseek_pts;
+    double tolerance = mpctx->hrseek_backstep ? 0 : .005;
+    if (mpctx->video_status == STATUS_SYNCING) {
+        hrseek = mpctx->hrseek_active;
+        // playback_pts is normally only set when audio and video have started
+        // playing normally. If video is in syncing mode, then this must mean
+        // video was just enabled via track switching - skip to current time.
+        if (!hrseek && mpctx->playback_pts != MP_NOPTS_VALUE) {
+            hrseek = true;
+            hrseek_pts = mpctx->playback_pts;
+        }
+    }
 
     if (vo_c->is_coverart) {
         if (vo_has_frame(mpctx->video_out))
@@ -495,17 +507,11 @@ static int video_output_image(struct MPContext *mpctx)
                 mp_pin_out_unread(vo_c->filter->f->pins[1], frame);
                 img = NULL;
                 r = VD_EOF;
-            } else if (hrseek && mpctx->hrseek_lastframe) {
-                mp_image_setrefp(&mpctx->saved_frame, img);
-            } else if (hrseek && img->pts < mpctx->hrseek_pts - .005) {
-                /* just skip - but save if backstep active */
-                if (mpctx->hrseek_backstep)
-                    mp_image_setrefp(&mpctx->saved_frame, img);
-            } else if (mpctx->video_status == STATUS_SYNCING &&
-                       mpctx->playback_pts != MP_NOPTS_VALUE &&
-                       img->pts < mpctx->playback_pts && !vo_c->is_coverart)
+            } else if (hrseek && (img->pts < hrseek_pts - tolerance ||
+                                  mpctx->hrseek_lastframe))
             {
-                /* skip after stream-switching */
+                /* just skip - but save in case it was the last frame */
+                mp_image_setrefp(&mpctx->saved_frame, img);
             } else {
                 if (hrseek && mpctx->hrseek_backstep) {
                     if (mpctx->saved_frame) {
@@ -523,11 +529,12 @@ static int video_output_image(struct MPContext *mpctx)
         }
     }
 
-    // Last-frame seek
-    if (r <= 0 && hrseek && mpctx->hrseek_lastframe && mpctx->saved_frame) {
+    // If hr-seek went past EOF, use the last frame.
+    if (r <= 0 && hrseek && mpctx->saved_frame && r == VD_EOF) {
         add_new_frame(mpctx, mpctx->saved_frame);
         mpctx->saved_frame = NULL;
-        r = VD_PROGRESS;
+        r = VD_EOF;
+        *logical_eof = true;
     }
 
     return have_new_frame(mpctx, r <= 0) ? VD_NEW_FRAME : r;
@@ -556,7 +563,7 @@ static void update_avsync_before_frame(struct MPContext *mpctx)
     struct MPOpts *opts = mpctx->opts;
     struct vo *vo = mpctx->video_out;
 
-    if (mpctx->vo_chain->is_coverart || mpctx->video_status < STATUS_READY) {
+    if (mpctx->video_status < STATUS_READY) {
         mpctx->time_frame = 0;
     } else if (mpctx->display_sync_active || opts->video_sync == VS_NONE) {
         // don't touch the timing
@@ -780,6 +787,12 @@ static void handle_display_sync_frame(struct MPContext *mpctx,
 
     mpctx->display_sync_active = false;
 
+    if (mode == VS_DISP_ADROP && !mpctx->audio_drop_deprecated_msg) {
+        MP_WARN(mpctx, "video-sync=display-adrop mode is deprecated and will "
+                       "be removed in the future.\n");
+        mpctx->audio_drop_deprecated_msg = true;
+    }
+
     if (!VS_IS_DISP(mode))
         return;
     bool resample = mode == VS_DISP_RESAMPLE || mode == VS_DISP_RESAMPLE_VDROP ||
@@ -992,7 +1005,8 @@ void write_video(struct MPContext *mpctx)
     if (mpctx->paused && mpctx->video_status >= STATUS_READY)
         return;
 
-    int r = video_output_image(mpctx);
+    bool logical_eof = false;
+    int r = video_output_image(mpctx, &logical_eof);
     MP_TRACE(mpctx, "video_output_image: %d\n", r);
 
     if (r < 0)
@@ -1000,8 +1014,12 @@ void write_video(struct MPContext *mpctx)
 
     if (r == VD_WAIT) {
         // Heuristic to detect underruns.
-        if (mpctx->video_status == STATUS_PLAYING && !vo_still_displaying(vo))
+        if (mpctx->video_status == STATUS_PLAYING && !vo_still_displaying(vo) &&
+            !vo_c->underrun_signaled)
+        {
             vo_c->underrun = true;
+            vo_c->underrun_signaled = true;
+        }
         // Demuxer will wake us up for more packets to decode.
         return;
     }
@@ -1179,7 +1197,17 @@ void write_video(struct MPContext *mpctx)
 
     mp_notify(mpctx, MPV_EVENT_TICK, NULL);
 
-    if (mpctx->vo_chain->is_coverart)
+    if (vo_c->filter->got_output_eof && !mpctx->num_next_frames &&
+        mpctx->ao_chain)
+    {
+        MP_VERBOSE(mpctx, "assuming this was the last video frame\n");
+        // The main point of doing this is to prevent use of this for the
+        // playback_pts if audio is still running (=> seek behavior).
+        mpctx->video_status = STATUS_EOF;
+    }
+
+    // hr-seek past EOF -> returns last frame, but terminates playback.
+    if (logical_eof)
         mpctx->video_status = STATUS_EOF;
 
     if (mpctx->video_status != STATUS_EOF) {
@@ -1194,6 +1222,8 @@ void write_video(struct MPContext *mpctx)
             mpctx->max_frames--;
     }
 
+    vo_c->underrun_signaled = false;
+
     screenshot_flip(mpctx);
 
     mp_wakeup_core(mpctx);
