@@ -28,16 +28,27 @@
 
 #include "misc/bstr.h"
 
+// Minimum guaranteed buffer and seek-back size. For any reads <= of this size,
+// it's guaranteed that you can seek back by <= of this size again.
 #define STREAM_BUFFER_SIZE 2048
 
-// stream->mode
-#define STREAM_READ  0
-#define STREAM_WRITE 1
-
 // flags for stream_open_ext (this includes STREAM_READ and STREAM_WRITE)
-#define STREAM_SAFE_ONLY 4
-#define STREAM_NETWORK_ONLY 8
-#define STREAM_SILENT 16
+
+// stream->mode
+#define STREAM_READ             0
+#define STREAM_WRITE            (1 << 0)
+
+#define STREAM_SILENT           (1 << 1)
+
+// Origin value for "security". This is an integer within the flags bit-field.
+#define STREAM_ORIGIN_DIRECT    (1 << 2) // passed from cmdline or loadfile
+#define STREAM_ORIGIN_FS        (2 << 2) // referenced from playlist on unix FS
+#define STREAM_ORIGIN_NET       (3 << 2) // referenced from playlist on network
+#define STREAM_ORIGIN_UNSAFE    (4 << 2) // from a grotesque source
+
+#define STREAM_ORIGIN_MASK      (7 << 2) // for extracting origin value from flags
+
+// end flags for stream_open_ext (the naming convention sucks)
 
 #define STREAM_UNSAFE -3
 #define STREAM_NO_MATCH -2
@@ -46,8 +57,6 @@
 #define STREAM_OK    1
 
 enum stream_ctrl {
-    STREAM_CTRL_GET_SIZE = 1,
-
     // Certain network protocols
     STREAM_CTRL_AVSEEK,
     STREAM_CTRL_HAS_AVSEEK,
@@ -100,29 +109,30 @@ typedef struct stream_info_st {
     int (*open2)(struct stream *st, struct stream_open_args *args);
     const char *const *protocols;
     bool can_write;     // correctly checks for READ/WRITE modes
-    bool is_safe;       // opening is no security issue, even with remote provided URLs
-    bool is_network;    // used to restrict remote playlist entries to remote URLs
+    int stream_origin;  // 0 or set of STREAM_ORIGIN_*; if 0, the same origin
+                        // is set, or the stream's open() function handles it
 } stream_info_t;
 
 typedef struct stream {
     const struct stream_info_st *info;
 
     // Read
-    int (*fill_buffer)(struct stream *s, char *buffer, int max_len);
+    int (*fill_buffer)(struct stream *s, void *buffer, int max_len);
     // Write
-    int (*write_buffer)(struct stream *s, char *buffer, int len);
+    int (*write_buffer)(struct stream *s, void *buffer, int len);
     // Seek
     int (*seek)(struct stream *s, int64_t pos);
+    // Total stream size in bytes (negative if unavailable)
+    int64_t (*get_size)(struct stream *s);
     // Control
     int (*control)(struct stream *s, int cmd, void *arg);
     // Close
     void (*close)(struct stream *s);
 
-    int read_chunk; // maximum amount of data to read at once to limit latency
-    unsigned int buf_pos, buf_len;
     int64_t pos;
-    int eof;
+    int eof; // valid only after read calls that returned a short result
     int mode; //STREAM_READ or STREAM_WRITE
+    int stream_origin; // any STREAM_ORIGIN_*
     void *priv; // used for DVD, TV, RTSP etc
     char *url;  // filename/url (possibly including protocol prefix)
     char *path; // filename (url without protocol prefix)
@@ -132,7 +142,7 @@ typedef struct stream {
     bool streaming : 1; // known to be a network stream if true
     bool seekable : 1; // presence of general byte seeking support
     bool fast_skip : 1; // consider stream fast enough to fw-seek by skipping
-    bool is_network : 1; // original stream_info_t.is_network flag
+    bool is_network : 1; // I really don't know what this is for
     bool is_local_file : 1; // from the filesystem
     bool is_directory : 1; // directory on the filesystem
     bool access_references : 1; // open other streams
@@ -144,42 +154,61 @@ typedef struct stream {
     // Read statistic for fill_buffer calls. All bytes read by fill_buffer() are
     // added to this. The user can reset this as needed.
     uint64_t total_unbuffered_read_bytes;
+    // Seek statistics. The user can reset this as needed.
+    uint64_t total_stream_seeks;
 
+    // Buffer size requested by user; s->buffer may have a different size
+    int requested_buffer_size;
+
+    // This is a ring buffer. It is reset only on seeks (or when buffers are
+    // dropped). Otherwise old contents always stay valid.
+    // The valid buffer is from buf_start to buf_end; buf_end can be larger
+    // than the buffer size (requires wrap around). buf_cur is a value in the
+    // range [buf_start, buf_end].
+    // When reading more data from the stream, buf_start is advanced as old
+    // data is overwritten with new data.
+    // Example:
+    //    0  1  2  3    4  5  6  7    8  9  10 11  12  13 14 15
+    //  +===========================+---------------------------+
+    //  + 05 06 07 08 | 01 02 03 04 + 05 06 07 08 | 01 02 03 04 +
+    //  +===========================+---------------------------+
+    //                  ^ buf_start (4)  |          |
+    //                                   |          ^ buf_end (12 % 8 => 4)
+    //                                   ^ buf_cur (9 % 8 => 1)
+    // Here, the entire 8 byte buffer is filled, i.e. buf_end - buf_start = 8.
+    // buffer_mask == 7, so (x & buffer_mask) == (x % buffer_size)
+    unsigned int buf_start; // index of oldest byte in buffer (is <= buffer_mask)
+    unsigned int buf_cur;   // current read pos (can be > buffer_mask)
+    unsigned int buf_end;   // end position (can be > buffer_mask)
+
+    unsigned int buffer_mask; // buffer_size-1, where buffer_size == 2**n
     uint8_t *buffer;
-
-    int buffer_alloc;
-    uint8_t buffer_inline[STREAM_BUFFER_SIZE];
 } stream_t;
 
-int stream_fill_buffer(stream_t *s);
+// Non-inline version of stream_read_char().
+int stream_read_char_fallback(stream_t *s);
 
-int stream_write_buffer(stream_t *s, unsigned char *buf, int len);
+int stream_write_buffer(stream_t *s, void *buf, int len);
 
 inline static int stream_read_char(stream_t *s)
 {
-    return (s->buf_pos < s->buf_len) ? s->buffer[s->buf_pos++] :
-           (stream_fill_buffer(s) ? s->buffer[s->buf_pos++] : -256);
+    return s->buf_cur < s->buf_end
+        ? s->buffer[(s->buf_cur++) & s->buffer_mask]
+        : stream_read_char_fallback(s);
 }
 
-unsigned char *stream_read_line(stream_t *s, unsigned char *mem, int max,
-                                int utf16);
 int stream_skip_bom(struct stream *s);
 
-inline static int stream_eof(stream_t *s)
-{
-    return s->eof;
-}
-
 inline static int64_t stream_tell(stream_t *s)
 {
-    return s->pos + s->buf_pos - s->buf_len;
+    return s->pos + s->buf_cur - s->buf_end;
 }
 
-bool stream_skip(stream_t *s, int64_t len);
+bool stream_seek_skip(stream_t *s, int64_t pos);
 bool stream_seek(stream_t *s, int64_t pos);
-int stream_read(stream_t *s, char *mem, int total);
-int stream_read_partial(stream_t *s, char *buf, int buf_size);
-struct bstr stream_peek(stream_t *s, int len);
+int stream_read(stream_t *s, void *mem, int total);
+int stream_read_partial(stream_t *s, void *buf, int buf_size);
+int stream_read_peek(stream_t *s, void *buf, int buf_size);
 void stream_drop_buffers(stream_t *s);
 int64_t stream_get_size(stream_t *s);
 
@@ -204,7 +233,6 @@ struct stream_open_args {
 int stream_create_with_args(struct stream_open_args *args, struct stream **ret);
 struct stream *stream_create(const char *url, int flags,
                              struct mp_cancel *c, struct mpv_global *global);
-struct stream *stream_open(const char *filename, struct mpv_global *global);
 stream_t *open_output_stream(const char *filename, struct mpv_global *global);
 
 void mp_url_unescape_inplace(char *buf);
@@ -224,6 +252,7 @@ char *mp_file_get_path(void *talloc_ctx, bstr url);
 // stream_lavf.c
 struct AVDictionary;
 void mp_setup_av_network_options(struct AVDictionary **dict,
+                                 const char *target_fmt,
                                  struct mpv_global *global,
                                  struct mp_log *log);
 
