@@ -46,22 +46,13 @@ static void smb_auth_fn(const char *server, const char *share,
   workgroup[wgmaxlen - 1] = '\0';
 }
 
-static int control(stream_t *s, int cmd, void *arg) {
+static int64_t get_size(stream_t *s) {
   struct priv *p = s->priv;
-  switch(cmd) {
-    case STREAM_CTRL_GET_SIZE: {
-      pthread_mutex_lock(&smb_lock);
-      off_t size = smbc_lseek(p->fd,0,SEEK_END);
-      smbc_lseek(p->fd,s->pos,SEEK_SET);
-      pthread_mutex_unlock(&smb_lock);
-      if(size != (off_t)-1) {
-        *(int64_t *)arg = size;
-        return 1;
-      }
-    }
-    break;
-  }
-  return STREAM_UNSUPPORTED;
+  pthread_mutex_lock(&smb_lock);
+  off_t size = smbc_lseek(p->fd,0,SEEK_END);
+  smbc_lseek(p->fd,s->pos,SEEK_SET);
+  pthread_mutex_unlock(&smb_lock);
+  return size;
 }
 
 static int seek(stream_t *s,int64_t newpos) {
@@ -75,7 +66,7 @@ static int seek(stream_t *s,int64_t newpos) {
   return 1;
 }
 
-static int fill_buffer(stream_t *s, char* buffer, int max_len){
+static int fill_buffer(stream_t *s, void *buffer, int max_len){
   struct priv *p = s->priv;
   pthread_mutex_lock(&smb_lock);
   int r = smbc_read(p->fd,buffer,max_len);
@@ -83,7 +74,7 @@ static int fill_buffer(stream_t *s, char* buffer, int max_len){
   return (r <= 0) ? -1 : r;
 }
 
-static int write_buffer(stream_t *s, char* buffer, int len) {
+static int write_buffer(stream_t *s, void *buffer, int len) {
   struct priv *p = s->priv;
   int wr;
   pthread_mutex_lock(&smb_lock);
@@ -149,8 +140,7 @@ static int open_f (stream_t *stream)
   stream->fill_buffer = fill_buffer;
   stream->write_buffer = write_buffer;
   stream->close = close_f;
-  stream->control = control;
-  stream->read_chunk = 128 * 1024;
+  stream->get_size = get_size;
   stream->streaming = true;
 
   return STREAM_OK;
@@ -161,4 +151,5 @@ const stream_info_t stream_info_smb = {
     .open = open_f,
     .protocols = (const char*const[]){"smb", NULL},
     .can_write = true, //who's gonna do that?
+    .stream_origin = STREAM_ORIGIN_FS,
 };
