@@ -31,6 +31,7 @@
 #include "common/global.h"
 #include "common/msg.h"
 #include "common/recorder.h"
+#include "misc/dispatch.h"
 #include "osdep/threads.h"
 
 extern const struct sd_functions sd_ass;
@@ -57,6 +58,7 @@ struct dec_sub {
     struct attachment_list *attachments;
 
     struct sh_stream *sh;
+    int play_dir;
     double last_pkt_pts;
     bool preload_attempted;
     double video_fps;
@@ -69,6 +71,8 @@ struct dec_sub {
     struct sd *sd;
 
     struct demux_packet *new_segment;
+
+    struct mp_dispatch_queue *demux_waiter;
 };
 
 static void update_subtitle_speed(struct dec_sub *sub)
@@ -94,7 +98,7 @@ static double pts_to_subtitle(struct dec_sub *sub, double pts)
     struct mp_subtitle_opts *opts = sub->opts;
 
     if (pts != MP_NOPTS_VALUE)
-        pts = (pts - opts->sub_delay) / sub->sub_speed;
+        pts = (pts * sub->play_dir - opts->sub_delay) / sub->sub_speed;
 
     return pts;
 }
@@ -104,7 +108,7 @@ static double pts_from_subtitle(struct dec_sub *sub, double pts)
     struct mp_subtitle_opts *opts = sub->opts;
 
     if (pts != MP_NOPTS_VALUE)
-        pts = pts * sub->sub_speed + opts->sub_delay;
+        pts = (pts * sub->sub_speed + opts->sub_delay) * sub->play_dir;
 
     return pts;
 }
@@ -119,12 +123,21 @@ void sub_unlock(struct dec_sub *sub)
     pthread_mutex_unlock(&sub->lock);
 }
 
+static void wakeup_demux(void *ctx)
+{
+    struct mp_dispatch_queue *q = ctx;
+    mp_dispatch_interrupt(q);
+}
+
 void sub_destroy(struct dec_sub *sub)
 {
     if (!sub)
         return;
-    sub_reset(sub);
-    sub->sd->driver->uninit(sub->sd);
+    demux_set_stream_wakeup_cb(sub->sh, NULL, NULL);
+    if (sub->sd) {
+        sub_reset(sub);
+        sub->sd->driver->uninit(sub->sd);
+    }
     talloc_free(sub->sd);
     pthread_mutex_destroy(&sub->lock);
     talloc_free(sub);
@@ -174,21 +187,25 @@ struct dec_sub *sub_create(struct mpv_global *global, struct sh_stream *sh,
         .sh = sh,
         .codec = sh->codec,
         .attachments = talloc_steal(sub, attachments),
+        .play_dir = 1,
         .last_pkt_pts = MP_NOPTS_VALUE,
         .last_vo_pts = MP_NOPTS_VALUE,
         .start = MP_NOPTS_VALUE,
         .end = MP_NOPTS_VALUE,
+        .demux_waiter = mp_dispatch_create(sub),
     };
     sub->opts = sub->opts_cache->opts;
     mpthread_mutex_init_recursive(&sub->lock);
 
+    demux_set_stream_wakeup_cb(sub->sh, wakeup_demux, sub->demux_waiter);
+
     sub->sd = init_decoder(sub);
     if (sub->sd) {
         update_subtitle_speed(sub);
         return sub;
     }
 
-    talloc_free(sub);
+    sub_destroy(sub);
     return NULL;
 }
 
@@ -237,7 +254,12 @@ void sub_preload(struct dec_sub *sub)
     sub->preload_attempted = true;
 
     for (;;) {
-        struct demux_packet *pkt = demux_read_packet(sub->sh);
+        struct demux_packet *pkt = NULL;
+        int r = demux_read_packet_async(sub->sh, &pkt);
+        if (r == 0) {
+            mp_dispatch_queue_process(sub->demux_waiter, INFINITY);
+            continue;
+        }
         if (!pkt)
             break;
         sub->sd->driver->decode(sub->sd, pkt);
@@ -337,7 +359,6 @@ void sub_get_bitmaps(struct dec_sub *sub, struct mp_osd_res dim, int format,
 char *sub_get_text(struct dec_sub *sub, double pts)
 {
     pthread_mutex_lock(&sub->lock);
-    struct mp_subtitle_opts *opts = sub->opts;
     char *text = NULL;
 
     pts = pts_to_subtitle(sub, pts);
@@ -345,12 +366,29 @@ char *sub_get_text(struct dec_sub *sub, double pts)
     sub->last_vo_pts = pts;
     update_segment(sub);
 
-    if (opts->sub_visibility && sub->sd->driver->get_text)
+    if (sub->sd->driver->get_text)
         text = sub->sd->driver->get_text(sub->sd, pts);
     pthread_mutex_unlock(&sub->lock);
     return text;
 }
 
+struct sd_times sub_get_times(struct dec_sub *sub, double pts)
+{
+    pthread_mutex_lock(&sub->lock);
+    struct sd_times res = { .start = MP_NOPTS_VALUE, .end = MP_NOPTS_VALUE };
+
+    pts = pts_to_subtitle(sub, pts);
+
+    sub->last_vo_pts = pts;
+    update_segment(sub);
+
+    if (sub->sd->driver->get_times)
+        res = sub->sd->driver->get_times(sub->sd, pts);
+
+    pthread_mutex_unlock(&sub->lock);
+    return res;
+}
+
 void sub_reset(struct dec_sub *sub)
 {
     pthread_mutex_lock(&sub->lock);
@@ -413,3 +451,9 @@ void sub_set_recorder_sink(struct dec_sub *sub, struct mp_recorder_sink *sink)
     pthread_mutex_unlock(&sub->lock);
 }
 
+void sub_set_play_dir(struct dec_sub *sub, int dir)
+{
+    pthread_mutex_lock(&sub->lock);
+    sub->play_dir = dir;
+    pthread_mutex_unlock(&sub->lock);
+}
