@@ -242,8 +242,8 @@ static void check_obj_resize(struct osd_state *osd, struct mp_osd_res res,
 {
     if (!osd_res_equals(res, obj->vo_res)) {
         obj->vo_res = res;
-        mp_client_broadcast_event(mp_client_api_get_core(osd->global->client_api),
-                                  MP_EVENT_WIN_RESIZE, NULL);
+        mp_client_broadcast_event_external(osd->global->client_api,
+                                           MP_EVENT_WIN_RESIZE, NULL);
     }
 }
 
