@@ -15,13 +15,12 @@
  * License along with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 
-#include <libavutil/common.h>
-
 #include "config.h"
 
 #include "mpv_talloc.h"
@@ -81,8 +80,8 @@ static void destroy_ass_renderer(struct ass_state *ass)
 
 static void destroy_external(struct osd_external *ext)
 {
-    talloc_free(ext->text);
     destroy_ass_renderer(&ext->ass);
+    talloc_free(ext);
 }
 
 void osd_destroy_backend(struct osd_state *osd)
@@ -91,7 +90,7 @@ void osd_destroy_backend(struct osd_state *osd)
         struct osd_object *obj = osd->objs[n];
         destroy_ass_renderer(&obj->ass);
         for (int i = 0; i < obj->num_externals; i++)
-            destroy_external(&obj->externals[i]);
+            destroy_external(obj->externals[i]);
         obj->num_externals = 0;
     }
 }
@@ -190,6 +189,7 @@ void osd_get_function_sym(char *buffer, size_t buffer_size, int osd_function)
 
 static void mangle_ass(bstr *dst, const char *in)
 {
+    const char *start = in;
     bool escape_ass = true;
     while (*in) {
         // As used by osd_get_function_sym().
@@ -207,6 +207,12 @@ static void mangle_ass(bstr *dst, const char *in)
         }
         if (escape_ass && *in == '{')
             bstr_xappend(NULL, dst, bstr0("\\"));
+        // Libass will strip leading whitespace
+        if (in[0] == ' ' && (in == start || in[-1] == '\n')) {
+            bstr_xappend(NULL, dst, bstr0("\\h"));
+            in += 1;
+            continue;
+        }
         bstr_xappend(NULL, dst, (bstr){(char *)in, 1});
         // Break ASS escapes with U+2060 WORD JOINER
         if (escape_ass && *in == '\\')
@@ -227,7 +233,7 @@ static ASS_Event *add_osd_ass_event_escaped(ASS_Track *track, const char *style,
 
 static ASS_Style *prepare_osd_ass(struct osd_state *osd, struct osd_object *obj)
 {
-    struct MPOpts *opts = osd->opts;
+    struct mp_osd_render_opts *opts = osd->opts;
 
     create_ass_track(osd, obj, &obj->ass);
 
@@ -281,7 +287,7 @@ struct ass_draw {
 
 static void ass_draw_start(struct ass_draw *d)
 {
-    d->scale = FFMAX(d->scale, 1);
+    d->scale = MPMAX(d->scale, 1);
     d->text = talloc_asprintf_append(d->text, "{\\p%d}", d->scale);
 }
 
@@ -342,7 +348,7 @@ static void get_osd_bar_box(struct osd_state *osd, struct osd_object *obj,
                             float *o_x, float *o_y, float *o_w, float *o_h,
                             float *o_border)
 {
-    struct MPOpts *opts = osd->opts;
+    struct mp_osd_render_opts *opts = osd->opts;
 
     create_ass_track(osd, obj, &obj->ass);
     ASS_Track *track = obj->ass.track;
@@ -361,9 +367,9 @@ static void get_osd_bar_box(struct osd_state *osd, struct osd_object *obj,
     float base_size = 0.03125;
     style->Outline *= *o_h / track->PlayResY / base_size;
     // So that the chapter marks have space between them
-    style->Outline = FFMIN(style->Outline, *o_h / 5.2);
+    style->Outline = MPMIN(style->Outline, *o_h / 5.2);
     // So that the border is not 0
-    style->Outline = FFMAX(style->Outline, *o_h / 32.0);
+    style->Outline = MPMAX(style->Outline, *o_h / 32.0);
     // Rendering with shadow is broken (because there's more than one shape)
     style->Shadow = 0;
 
@@ -464,11 +470,9 @@ static void update_osd(struct osd_state *osd, struct osd_object *obj)
 static void update_external(struct osd_state *osd, struct osd_object *obj,
                             struct osd_external *ext)
 {
-    bstr t = bstr0(ext->text);
-    if (!t.len)
-        return;
-    ext->ass.res_x = ext->res_x;
-    ext->ass.res_y = ext->res_y;
+    bstr t = bstr0(ext->ov.data);
+    ext->ass.res_x = ext->ov.res_x;
+    ext->ass.res_y = ext->ov.res_y;
     create_ass_track(osd, obj, &ext->ass);
 
     clear_ass(&ext->ass);
@@ -491,29 +495,42 @@ static void update_external(struct osd_state *osd, struct osd_object *obj,
     }
 }
 
-void osd_set_external(struct osd_state *osd, void *id, int res_x, int res_y,
-                      char *text)
+static int cmp_zorder(const void *pa, const void *pb)
+{
+    const struct osd_external *a = *(struct osd_external **)pa;
+    const struct osd_external *b = *(struct osd_external **)pb;
+    return a->ov.z == b->ov.z ? 0 : (a->ov.z > b->ov.z ? 1 : -1);
+}
+
+void osd_set_external(struct osd_state *osd, struct osd_external_ass *ov)
 {
     pthread_mutex_lock(&osd->lock);
     struct osd_object *obj = osd->objs[OSDTYPE_EXTERNAL];
-    struct osd_external *entry = 0;
+    bool zorder_changed = false;
+    int index = -1;
+
     for (int n = 0; n < obj->num_externals; n++) {
-        if (obj->externals[n].id == id) {
-            entry = &obj->externals[n];
+        struct osd_external *e = obj->externals[n];
+        if (e->ov.id == ov->id && e->ov.owner == ov->owner) {
+            index = n;
             break;
         }
     }
-    if (!entry && !text)
-        goto done;
 
-    if (!entry) {
-        struct osd_external new = { .id = id };
+    if (index < 0) {
+        if (!ov->format)
+            goto done;
+        struct osd_external *new = talloc_zero(NULL, struct osd_external);
+        new->ov.owner = ov->owner;
+        new->ov.id = ov->id;
         MP_TARRAY_APPEND(obj, obj->externals, obj->num_externals, new);
-        entry = &obj->externals[obj->num_externals - 1];
+        index = obj->num_externals - 1;
+        zorder_changed = true;
     }
 
-    if (!text) {
-        int index = entry - &obj->externals[0];
+    struct osd_external *entry = obj->externals[index];
+
+    if (!ov->format) {
         destroy_external(entry);
         MP_TARRAY_REMOVE_AT(obj->externals, obj->num_externals, index);
         obj->changed = true;
@@ -521,22 +538,46 @@ void osd_set_external(struct osd_state *osd, void *id, int res_x, int res_y,
         goto done;
     }
 
-    if (!entry->text || strcmp(entry->text, text) != 0 ||
-        entry->res_x != res_x || entry->res_y != res_y)
-    {
-        talloc_free(entry->text);
-        entry->text = talloc_strdup(NULL, text);
-        entry->res_x = res_x;
-        entry->res_y = res_y;
-        update_external(osd, obj, entry);
-        obj->changed = true;
-        osd->want_redraw_notification = true;
+    entry->ov.format = ov->format;
+    if (!entry->ov.data)
+        entry->ov.data = talloc_strdup(entry, "");
+    entry->ov.data[0] = '\0'; // reuse memory allocation
+    entry->ov.data = talloc_strdup_append(entry->ov.data, ov->data);
+    entry->ov.res_x = ov->res_x;
+    entry->ov.res_y = ov->res_y;
+    zorder_changed |= entry->ov.z != ov->z;
+    entry->ov.z = ov->z;
+
+    update_external(osd, obj, entry);
+
+    obj->changed = true;
+    osd->want_redraw_notification = true;
+
+    if (zorder_changed) {
+        qsort(obj->externals, obj->num_externals, sizeof(obj->externals[0]),
+              cmp_zorder);
     }
 
 done:
     pthread_mutex_unlock(&osd->lock);
 }
 
+void osd_set_external_remove_owner(struct osd_state *osd, void *owner)
+{
+    pthread_mutex_lock(&osd->lock);
+    struct osd_object *obj = osd->objs[OSDTYPE_EXTERNAL];
+    for (int n = obj->num_externals - 1; n >= 0; n--) {
+        struct osd_external *e = obj->externals[n];
+        if (e->ov.owner == owner) {
+            destroy_external(e);
+            MP_TARRAY_REMOVE_AT(obj->externals, obj->num_externals, n);
+            obj->changed = true;
+            osd->want_redraw_notification = true;
+        }
+    }
+    pthread_mutex_unlock(&osd->lock);
+}
+
 static void append_ass(struct ass_state *ass, struct mp_osd_res *res,
                        ASS_Image **img_list, bool *changed)
 {
@@ -568,8 +609,8 @@ void osd_object_get_bitmaps(struct osd_state *osd, struct osd_object *obj,
 
     append_ass(&obj->ass, &obj->vo_res, &obj->ass_imgs[0], &obj->changed);
     for (int n = 0; n < obj->num_externals; n++) {
-        append_ass(&obj->externals[n].ass, &obj->vo_res, &obj->ass_imgs[n + 1],
-                   &obj->changed);
+        append_ass(&obj->externals[n]->ass, &obj->vo_res,
+                   &obj->ass_imgs[n + 1], &obj->changed);
     }
 
     mp_ass_packer_pack(obj->ass_packer, obj->ass_imgs, obj->num_externals + 1,
