@@ -78,7 +78,10 @@ const struct m_opt_choice_alternatives mp_csp_trc_names[] = {
     {"srgb",        MP_CSP_TRC_SRGB},
     {"linear",      MP_CSP_TRC_LINEAR},
     {"gamma1.8",    MP_CSP_TRC_GAMMA18},
+    {"gamma2.0",    MP_CSP_TRC_GAMMA20},
     {"gamma2.2",    MP_CSP_TRC_GAMMA22},
+    {"gamma2.4",    MP_CSP_TRC_GAMMA24},
+    {"gamma2.6",    MP_CSP_TRC_GAMMA26},
     {"gamma2.8",    MP_CSP_TRC_GAMMA28},
     {"prophoto",    MP_CSP_TRC_PRO_PHOTO},
     {"pq",          MP_CSP_TRC_PQ},
@@ -708,8 +711,10 @@ void mp_get_csp_matrix(struct mp_csp_params *params, struct mp_cmat *m)
     }
     case MP_CSP_XYZ: {
         // The vo should probably not be using a matrix generated by this
-        // function for XYZ sources, but if it does, let's just assume it
-        // wants BT.709 with D65 white point (virtually all other content).
+        // function for XYZ sources, but if it does, let's just convert it to
+        // an equivalent RGB space based on the colorimetry metadata it
+        // provided in mp_csp_params. (At the risk of clipping, if the
+        // chosen primaries are too small to fit the actual data)
         mp_get_xyz2rgb_coeffs(params, MP_INTENT_RELATIVE_COLORIMETRIC, m);
         levels_in = -1;
         break;
