@@ -118,7 +118,8 @@ const struct m_sub_options vd_lavc_conf = {
                           0, 1, INT_MAX, ({"no", INT_MAX}, {"yes", 1})),
         OPT_KEYVALUELIST("vd-lavc-o", avopts, 0),
         OPT_FLAG("vd-lavc-dr", dr, 0),
-        OPT_STRING_VALIDATE("hwdec", hwdec_api, M_OPT_OPTIONAL_PARAM,
+        OPT_STRING_VALIDATE("hwdec", hwdec_api,
+                            M_OPT_OPTIONAL_PARAM | UPDATE_HWDEC,
                             hwdec_validate_opt),
         OPT_STRING("hwdec-codecs", hwdec_codecs, 0),
         OPT_IMAGEFORMAT("hwdec-image-format", hwdec_image_format, 0, .min = -1),
@@ -135,7 +136,7 @@ const struct m_sub_options vd_lavc_conf = {
         .skip_frame = AVDISCARD_DEFAULT,
         .framedrop = AVDISCARD_NONREF,
         .dr = 1,
-        .hwdec_api = HAVE_RPI ? "mmal" : "no",
+        .hwdec_api = "no",
         .hwdec_codecs = "h264,vc1,hevc,vp9",
         // Maximum number of surfaces the player wants to buffer. This number
         // might require adjustment depending on whatever the player does;
@@ -154,6 +155,7 @@ struct hwdec_info {
     enum AVPixelFormat pix_fmt; // if not NONE, select in get_format
     bool use_hw_frames; // set AVCodecContext.hw_frames_ctx
     bool use_hw_device; // set AVCodecContext.hw_device_ctx
+    unsigned int flags; // HWDEC_FLAG_*
 
     // for internal sorting
     int auto_pos;
@@ -172,7 +174,7 @@ typedef struct lavc_ctx {
     AVRational codec_timebase;
     enum AVDiscard skip_frame;
     bool flushing;
-    bool eof_returned;
+    struct lavc_state state;
     const char *decoder;
     bool hwdec_requested;
     bool hwdec_failed;
@@ -215,19 +217,33 @@ typedef struct lavc_ctx {
     struct mp_decoder public;
 } vd_ffmpeg_ctx;
 
+enum {
+    HWDEC_FLAG_AUTO         = (1 << 0), // prioritize in autoprobe order
+    HWDEC_FLAG_WHITELIST    = (1 << 1), // whitelist for auto-safe
+};
+
+struct autoprobe_info {
+    const char *method_name;
+    unsigned int flags;         // HWDEC_FLAG_*
+};
+
 // Things not included in this list will be tried last, in random order.
-static const char *const hwdec_autoprobe_order[] = {
-    "d3d11va",
-    "dxva2",
-    "dxva2-copy",
-    "d3d11va-copy",
-    "nvdec",
-    "nvdec-copy",
-    "vaapi",
-    "vaapi-copy",
-    "vdpau",
-    "vdpau-copy",
-    0
+const struct autoprobe_info hwdec_autoprobe_info[] = {
+    {"d3d11va",         HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"dxva2",           HWDEC_FLAG_AUTO},
+    {"dxva2-copy",      HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"d3d11va-copy",    HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"nvdec",           HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"nvdec-copy",      HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"vaapi",           HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"vaapi-copy",      HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"vdpau",           HWDEC_FLAG_AUTO},
+    {"vdpau-copy",      HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"mmal",            HWDEC_FLAG_AUTO},
+    {"mmal-copy",       HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"videotoolbox",    HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {"videotoolbox-copy", HWDEC_FLAG_AUTO | HWDEC_FLAG_WHITELIST},
+    {0}
 };
 
 static int hwdec_compare(const void *p1, const void *p2)
@@ -265,9 +281,14 @@ static void add_hwdec_item(struct hwdec_info **infos, int *num_infos,
 
     info.rank = *num_infos;
     info.auto_pos = INT_MAX;
-    for (int x = 0; hwdec_autoprobe_order[x]; x++) {
-        if (strcmp(hwdec_autoprobe_order[x], info.method_name) == 0)
-            info.auto_pos = x;
+
+    for (int x = 0; hwdec_autoprobe_info[x].method_name; x++) {
+        const struct autoprobe_info *entry = &hwdec_autoprobe_info[x];
+        if (strcmp(entry->method_name, info.method_name) == 0) {
+            info.flags |= entry->flags;
+            if (info.flags & HWDEC_FLAG_AUTO)
+                info.auto_pos = x;
+        }
     }
 
     MP_TARRAY_APPEND(NULL, *infos, *num_infos, info);
@@ -433,8 +454,11 @@ static void select_and_set_hwdec(struct mp_filter *vd)
     bool hwdec_auto_all = bstr_equals0(opt, "auto") ||
                           bstr_equals0(opt, "yes") ||
                           bstr_equals0(opt, "");
-    bool hwdec_auto_copy = bstr_equals0(opt, "auto-copy");
-    bool hwdec_auto = hwdec_auto_all || hwdec_auto_copy;
+    bool hwdec_auto_safe = bstr_equals0(opt, "auto-safe") ||
+                           bstr_equals0(opt, "auto-copy-safe");
+    bool hwdec_auto_copy = bstr_equals0(opt, "auto-copy") ||
+                           bstr_equals0(opt, "auto-copy-safe");
+    bool hwdec_auto = hwdec_auto_all || hwdec_auto_copy || hwdec_auto_safe;
 
     if (!hwdec_requested) {
         MP_VERBOSE(vd, "No hardware decoding requested.\n");
@@ -459,6 +483,9 @@ static void select_and_set_hwdec(struct mp_filter *vd)
                                  bstr_equals0(opt, hwdec->name)))
                 continue;
 
+            if (hwdec_auto_safe && !(hwdec->flags & HWDEC_FLAG_WHITELIST))
+                continue;
+
             MP_VERBOSE(vd, "Looking at hwdec %s...\n", hwdec->name);
 
             if (hwdec_auto_copy && !hwdec->copying) {
@@ -529,6 +556,12 @@ static int hwdec_validate_opt(struct mp_log *log, const m_option_t *opt,
 
         talloc_free(hwdecs);
 
+        mp_info(log, "  auto (yes '')\n");
+        mp_info(log, "  no\n");
+        mp_info(log, "  auto-safe\n");
+        mp_info(log, "  auto-copy\n");
+        mp_info(log, "  auto-copy-safe\n");
+
         return M_OPT_EXIT;
     }
     return 0;
@@ -927,14 +960,14 @@ fallback:
     return avcodec_default_get_buffer2(avctx, pic, flags);
 }
 
-static bool prepare_decoding(struct mp_filter *vd)
+static void prepare_decoding(struct mp_filter *vd)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
     AVCodecContext *avctx = ctx->avctx;
     struct vd_lavc_params *opts = ctx->opts;
 
-    if (!avctx || ctx->hwdec_failed)
-        return false;
+    if (!avctx)
+        return;
 
     int drop = ctx->framedrop_flags;
     if (drop == 1) {
@@ -950,8 +983,6 @@ static bool prepare_decoding(struct mp_filter *vd)
 
     if (ctx->hwdec_request_reinit)
         reset_avctx(vd);
-
-    return true;
 }
 
 static void handle_err(struct mp_filter *vd)
@@ -959,7 +990,8 @@ static void handle_err(struct mp_filter *vd)
     vd_ffmpeg_ctx *ctx = vd->priv;
     struct vd_lavc_params *opts = ctx->opts;
 
-    MP_WARN(vd, "Error while decoding frame!\n");
+    MP_WARN(vd, "Error while decoding frame%s!\n",
+            ctx->use_hwdec ? " (hardware decoding)" : "");
 
     if (ctx->use_hwdec) {
         ctx->hwdec_fail_count += 1;
@@ -968,88 +1000,108 @@ static void handle_err(struct mp_filter *vd)
     }
 }
 
-static bool do_send_packet(struct mp_filter *vd, struct demux_packet *pkt)
+static int send_packet(struct mp_filter *vd, struct demux_packet *pkt)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
     AVCodecContext *avctx = ctx->avctx;
 
-    if (!prepare_decoding(vd))
-        return false;
+    if (ctx->num_requeue_packets && ctx->requeue_packets[0] != pkt)
+        return AVERROR(EAGAIN); // cannot consume the packet
+
+    if (ctx->hwdec_failed)
+        return AVERROR(EAGAIN);
+
+    if (!ctx->avctx)
+        return AVERROR_EOF;
+
+    prepare_decoding(vd);
 
     if (avctx->skip_frame == AVDISCARD_ALL)
-        return true;
+        return 0;
 
     AVPacket avpkt;
     mp_set_av_packet(&avpkt, pkt, &ctx->codec_timebase);
 
     int ret = avcodec_send_packet(avctx, pkt ? &avpkt : NULL);
     if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF)
-        return false;
+        return ret;
 
-    if (ctx->hw_probing && ctx->num_sent_packets < 32) {
+    if (ctx->hw_probing && ctx->num_sent_packets < 32 &&
+        ctx->opts->software_fallback <= 32)
+    {
         pkt = pkt ? demux_copy_packet(pkt) : NULL;
         MP_TARRAY_APPEND(ctx, ctx->sent_packets, ctx->num_sent_packets, pkt);
     }
 
     if (ret < 0)
         handle_err(vd);
-    return true;
+    return ret;
 }
 
-static bool send_queued(struct mp_filter *vd)
+static void send_queued_packet(struct mp_filter *vd)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
 
-    while (ctx->num_requeue_packets && do_send_packet(vd, ctx->requeue_packets[0]))
-    {
+    assert(ctx->num_requeue_packets);
+    assert(!ctx->hw_probing);
+
+    if (send_packet(vd, ctx->requeue_packets[0]) != AVERROR(EAGAIN)) {
         talloc_free(ctx->requeue_packets[0]);
         MP_TARRAY_REMOVE_AT(ctx->requeue_packets, ctx->num_requeue_packets, 0);
     }
-
-    return ctx->num_requeue_packets == 0;
-}
-
-static bool send_packet(struct mp_filter *vd, struct demux_packet *pkt)
-{
-    if (!send_queued(vd))
-        return false;
-
-    return do_send_packet(vd, pkt);
 }
 
 // Returns whether decoder is still active (!EOF state).
-static bool decode_frame(struct mp_filter *vd)
+static int decode_frame(struct mp_filter *vd)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
     AVCodecContext *avctx = ctx->avctx;
 
-    if (!prepare_decoding(vd))
-        return true;
+    if (!avctx)
+        return AVERROR_EOF;
+
+    prepare_decoding(vd);
+
+    // Re-send old packets (typically after a hwdec fallback during init).
+    if (ctx->num_requeue_packets)
+        send_queued_packet(vd);
 
     int ret = avcodec_receive_frame(avctx, ctx->pic);
-    if (ret == AVERROR_EOF) {
-        // If flushing was initialized earlier and has ended now, make it start
-        // over in case we get new packets at some point in the future. This
-        // must take the delay queue into account, so avctx returns EOF until
-        // the delay queue has been drained.
-        if (!ctx->num_delay_queue)
-            reset_avctx(vd);
-        return false;
-    } else if (ret < 0 && ret != AVERROR(EAGAIN)) {
-        handle_err(vd);
+    if (ret < 0) {
+        if (ret == AVERROR_EOF) {
+            // If flushing was initialized earlier and has ended now, make it
+            // start over in case we get new packets at some point in the future.
+            // This must take the delay queue into account, so avctx returns EOF
+            // until the delay queue has been drained.
+            if (!ctx->num_delay_queue)
+                reset_avctx(vd);
+        } else if (ret == AVERROR(EAGAIN)) {
+            // just retry after caller writes a packet
+        } else {
+            handle_err(vd);
+        }
+        return ret;
     }
 
-    if (!ctx->pic->buf[0])
-        return true;
-
-    ctx->hwdec_fail_count = 0;
+    // If something was decoded successfully, it must return a frame with valid
+    // data.
+    assert(ctx->pic->buf[0]);
 
     struct mp_image *mpi = mp_image_from_av_frame(ctx->pic);
     if (!mpi) {
         av_frame_unref(ctx->pic);
-        return true;
+        return ret;
     }
-    assert(mpi->planes[0] || mpi->planes[3]);
+
+    if (mpi->imgfmt == IMGFMT_CUDA && !mpi->planes[0]) {
+        MP_ERR(vd, "CUDA frame without data. This is a FFmpeg bug.\n");
+        talloc_free(mpi);
+        handle_err(vd);
+        return AVERROR_BUG;
+    }
+
+    ctx->hwdec_fail_count = 0;
+
     mpi->pts = mp_pts_from_av(ctx->pic->pts, &ctx->codec_timebase);
     mpi->dts = mp_pts_from_av(ctx->pic->pkt_dts, &ctx->codec_timebase);
 
@@ -1061,14 +1113,14 @@ static bool decode_frame(struct mp_filter *vd)
     av_frame_unref(ctx->pic);
 
     MP_TARRAY_APPEND(ctx, ctx->delay_queue, ctx->num_delay_queue, mpi);
-    return true;
+    return ret;
 }
 
-static bool receive_frame(struct mp_filter *vd, struct mp_frame *out_frame)
+static int receive_frame(struct mp_filter *vd, struct mp_frame *out_frame)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
 
-    bool progress = decode_frame(vd);
+    int ret = decode_frame(vd);
 
     if (ctx->hwdec_failed) {
         // Failed hardware decoding? Try again in software.
@@ -1082,22 +1134,24 @@ static bool receive_frame(struct mp_filter *vd, struct mp_frame *out_frame)
         ctx->requeue_packets = pkts;
         ctx->num_requeue_packets = num_pkts;
 
-        send_queued(vd);
-        progress = decode_frame(vd);
+        return 0; // force retry
     }
 
-    if (!ctx->num_delay_queue)
-        return progress;
+    if (ret == AVERROR(EAGAIN) && ctx->num_requeue_packets)
+        return 0; // force retry, so send_queued_packet() gets called
 
-    if (ctx->num_delay_queue <= ctx->max_delay_queue && progress)
-        return true;
+    if (!ctx->num_delay_queue)
+        return ret;
+
+    if (ctx->num_delay_queue <= ctx->max_delay_queue && ret != AVERROR_EOF)
+        return AVERROR(EAGAIN);
 
     struct mp_image *res = ctx->delay_queue[0];
     MP_TARRAY_REMOVE_AT(ctx->delay_queue, ctx->num_delay_queue, 0);
 
     res = res ? mp_img_swap_to_native(res) : NULL;
     if (!res)
-        return progress;
+        return AVERROR_UNKNOWN;
 
     if (ctx->use_hwdec && ctx->hwdec.copying && res->hwctx) {
         struct mp_image *sw = mp_image_hw_download(res, ctx->hwdec_swpool);
@@ -1107,7 +1161,7 @@ static bool receive_frame(struct mp_filter *vd, struct mp_frame *out_frame)
             MP_ERR(vd, "Could not copy back hardware decoded frame.\n");
             ctx->hwdec_fail_count = INT_MAX - 1; // force fallback
             handle_err(vd);
-            return true;
+            return AVERROR_UNKNOWN;
         }
     }
 
@@ -1129,7 +1183,7 @@ static bool receive_frame(struct mp_filter *vd, struct mp_frame *out_frame)
     }
 
     *out_frame = MAKE_FRAME(MP_FRAME_VIDEO, res);
-    return true;
+    return 0;
 }
 
 static int control(struct mp_filter *vd, enum dec_ctrl cmd, void *arg)
@@ -1169,7 +1223,7 @@ static void process(struct mp_filter *vd)
 {
     vd_ffmpeg_ctx *ctx = vd->priv;
 
-    lavc_process(vd, &ctx->eof_returned, send_packet, receive_frame);
+    lavc_process(vd, &ctx->state, send_packet, receive_frame);
 }
 
 static void reset(struct mp_filter *vd)
@@ -1178,7 +1232,7 @@ static void reset(struct mp_filter *vd)
 
     flush_all(vd);
 
-    ctx->eof_returned = false;
+    ctx->state = (struct lavc_state){0};
     ctx->framedrop_flags = 0;
 }
 
