@@ -20,22 +20,21 @@
 #include <va/va.h>
 #include <va/va_vpp.h>
 
+#include <libavutil/hwcontext.h>
+#include <libavutil/hwcontext_vaapi.h>
+
 #include "config.h"
 #include "options/options.h"
-#include "vf.h"
+#include "filters/filter.h"
+#include "filters/filter_internal.h"
+#include "filters/user_filters.h"
 #include "refqueue.h"
+
+#include "video/fmt-conversion.h"
 #include "video/vaapi.h"
 #include "video/hwdec.h"
 #include "video/mp_image_pool.h"
 
-static bool check_error(struct vf_instance *vf, VAStatus status, const char *msg)
-{
-    if (status == VA_STATUS_SUCCESS)
-        return true;
-    MP_ERR(vf, "%s: %s\n", msg, vaErrorStr(status));
-    return false;
-}
-
 struct surface_refs {
     VASurfaceID *surfaces;
     int num_surfaces;
@@ -51,10 +50,14 @@ struct pipeline {
     struct surface_refs forward, backward;
 };
 
-struct vf_priv_s {
+struct opts {
     int deint_type;
     int interlaced_only;
     int reversal_bug;
+};
+
+struct priv {
+    struct opts *opts;
     bool do_deint;
     VABufferID buffers[VAProcFilterCount];
     int num_buffers;
@@ -62,23 +65,14 @@ struct vf_priv_s {
     VAContextID context;
     struct mp_image_params params;
     VADisplay display;
-    struct mp_vaapi_ctx *va;
+    AVBufferRef *av_device_ref;
     struct pipeline pipe;
-    struct mp_image_pool *pool;
-    int current_rt_format;
+    AVBufferRef *hw_pool;
 
     struct mp_refqueue *queue;
 };
 
-static const struct vf_priv_s vf_priv_default = {
-    .config = VA_INVALID_ID,
-    .context = VA_INVALID_ID,
-    .deint_type = 2,
-    .interlaced_only = 1,
-    .reversal_bug = 1,
-};
-
-static void add_surfaces(struct vf_priv_s *p, struct surface_refs *refs, int dir)
+static void add_surfaces(struct priv *p, struct surface_refs *refs, int dir)
 {
     for (int n = 0; n < refs->max_surfaces; n++) {
         struct mp_image *s = mp_refqueue_get(p->queue, (1 + n) * dir);
@@ -92,6 +86,7 @@ static void add_surfaces(struct vf_priv_s *p, struct surface_refs *refs, int dir
 }
 
 // The array items must match with the "deint" suboption values.
+// They're also sorted by quality.
 static const int deint_algorithm[] = {
     [0] = VAProcDeinterlacingNone,
     [1] = VAProcDeinterlacingBob, // first-field, special-cased
@@ -101,18 +96,18 @@ static const int deint_algorithm[] = {
     [5] = VAProcDeinterlacingMotionCompensated,
 };
 
-static void flush_frames(struct vf_instance *vf)
+static void flush_frames(struct mp_filter *f)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = f->priv;
     mp_refqueue_flush(p->queue);
 }
 
-static void update_pipeline(struct vf_instance *vf)
+static void update_pipeline(struct mp_filter *vf)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = vf->priv;
     VABufferID *filters = p->buffers;
     int num_filters = p->num_buffers;
-    if (p->deint_type && !p->do_deint) {
+    if (p->opts->deint_type && !p->do_deint) {
         filters++;
         num_filters--;
     }
@@ -130,7 +125,7 @@ static void update_pipeline(struct vf_instance *vf)
     };
     VAStatus status = vaQueryVideoProcPipelineCaps(p->display, p->context,
                                                    filters, num_filters, &caps);
-    if (!check_error(vf, status, "vaQueryVideoProcPipelineCaps()"))
+    if (!CHECK_VA_STATUS(vf, "vaQueryVideoProcPipelineCaps()"))
         goto nodeint;
     p->pipe.filters = filters;
     p->pipe.num_filters = num_filters;
@@ -138,7 +133,7 @@ static void update_pipeline(struct vf_instance *vf)
     p->pipe.num_output_colors = caps.num_output_color_standards;
     p->pipe.forward.max_surfaces = caps.num_forward_references;
     p->pipe.backward.max_surfaces = caps.num_backward_references;
-    if (p->reversal_bug) {
+    if (p->opts->reversal_bug) {
         int max = MPMAX(caps.num_forward_references, caps.num_backward_references);
         mp_refqueue_set_refs(p->queue, max, max);
     } else {
@@ -147,8 +142,8 @@ static void update_pipeline(struct vf_instance *vf)
     }
     mp_refqueue_set_mode(p->queue,
         (p->do_deint ? MP_MODE_DEINT : 0) |
-        (p->deint_type >= 2 ? MP_MODE_OUTPUT_FIELDS : 0) |
-        (p->interlaced_only ? MP_MODE_INTERLACED_ONLY : 0));
+        (p->opts->deint_type >= 2 ? MP_MODE_OUTPUT_FIELDS : 0) |
+        (p->opts->interlaced_only ? MP_MODE_INTERLACED_ONLY : 0));
     return;
 
 nodeint:
@@ -156,9 +151,47 @@ nodeint:
     mp_refqueue_set_mode(p->queue, 0);
 }
 
-static struct mp_image *render(struct vf_instance *vf)
+static struct mp_image *alloc_out(struct mp_filter *vf)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = vf->priv;
+
+    struct mp_image *fmt = mp_refqueue_get_format(p->queue);
+    if (!fmt || !fmt->hwctx)
+        return NULL;
+
+    AVHWFramesContext *hw_frames = (void *)fmt->hwctx->data;
+    // VAAPI requires the full surface size to match for input and output.
+    int src_w = hw_frames->width;
+    int src_h = hw_frames->height;
+
+    if (!mp_update_av_hw_frames_pool(&p->hw_pool, p->av_device_ref,
+                                     IMGFMT_VAAPI, IMGFMT_NV12, src_w, src_h))
+    {
+        MP_ERR(vf, "Failed to create hw pool.\n");
+        return NULL;
+    }
+
+    AVFrame *av_frame = av_frame_alloc();
+    if (!av_frame)
+        abort();
+    if (av_hwframe_get_buffer(p->hw_pool, av_frame, 0) < 0) {
+        MP_ERR(vf, "Failed to allocate frame from hw pool.\n");
+        av_frame_free(&av_frame);
+        return NULL;
+    }
+    struct mp_image *img = mp_image_from_av_frame(av_frame);
+    av_frame_free(&av_frame);
+    if (!img) {
+        MP_ERR(vf, "Unknown error.\n");
+        return NULL;
+    }
+    mp_image_set_size(img, fmt->w, fmt->h);
+    return img;
+}
+
+static struct mp_image *render(struct mp_filter *vf)
+{
+    struct priv *p = vf->priv;
 
     struct mp_image *in = mp_refqueue_get(p->queue, 0);
     struct mp_image *img = NULL;
@@ -170,12 +203,10 @@ static struct mp_image *render(struct vf_instance *vf)
     if (!p->pipe.filters || in_id == VA_INVALID_ID)
         goto cleanup;
 
-    int r_w, r_h;
-    va_surface_get_uncropped_size(in, &r_w, &r_h);
-    img = mp_image_pool_get(p->pool, IMGFMT_VAAPI, r_w, r_h);
+    img = alloc_out(vf);
     if (!img)
         goto cleanup;
-    mp_image_set_size(img, in->w, in->h);
+
     mp_image_copy_attributes(img, in);
 
     unsigned int flags = va_get_colorspace_flag(p->params.color.space);
@@ -192,7 +223,7 @@ static struct mp_image *render(struct vf_instance *vf)
         goto cleanup;
 
     VAStatus status = vaBeginPicture(p->display, p->context, id);
-    if (!check_error(vf, status, "vaBeginPicture()"))
+    if (!CHECK_VA_STATUS(vf, "vaBeginPicture()"))
         goto cleanup;
 
     need_end_picture = true;
@@ -201,12 +232,12 @@ static struct mp_image *render(struct vf_instance *vf)
     status = vaCreateBuffer(p->display, p->context,
                             VAProcPipelineParameterBufferType,
                             sizeof(*param), 1, NULL, &buffer);
-    if (!check_error(vf, status, "vaCreateBuffer()"))
+    if (!CHECK_VA_STATUS(vf, "vaCreateBuffer()"))
         goto cleanup;
 
     VAProcFilterParameterBufferDeinterlacing *filter_params;
     status = vaMapBuffer(p->display, *(p->pipe.filters), (void**)&filter_params);
-    if (!check_error(vf, status, "vaMapBuffer()"))
+    if (!CHECK_VA_STATUS(vf, "vaMapBuffer()"))
         goto cleanup;
 
     filter_params->flags = flags & VA_TOP_FIELD ? 0 : VA_DEINTERLACING_BOTTOM_FIELD;
@@ -216,7 +247,7 @@ static struct mp_image *render(struct vf_instance *vf)
     vaUnmapBuffer(p->display, *(p->pipe.filters));
 
     status = vaMapBuffer(p->display, buffer, (void**)&param);
-    if (!check_error(vf, status, "vaMapBuffer()"))
+    if (!CHECK_VA_STATUS(vf, "vaMapBuffer()"))
         goto cleanup;
 
     *param = (VAProcPipelineParameterBuffer){0};
@@ -228,7 +259,7 @@ static struct mp_image *render(struct vf_instance *vf)
     param->filters = p->pipe.filters;
     param->num_filters = p->pipe.num_filters;
 
-    int dir = p->reversal_bug ? -1 : 1;
+    int dir = p->opts->reversal_bug ? -1 : 1;
 
     add_surfaces(p, &p->pipe.forward, 1 * dir);
     param->forward_references = p->pipe.forward.surfaces;
@@ -247,7 +278,7 @@ static struct mp_image *render(struct vf_instance *vf)
     vaUnmapBuffer(p->display, buffer);
 
     status = vaRenderPicture(p->display, p->context, &buffer, 1);
-    if (!check_error(vf, status, "vaRenderPicture()"))
+    if (!CHECK_VA_STATUS(vf, "vaRenderPicture()"))
         goto cleanup;
 
     success = true;
@@ -262,163 +293,83 @@ cleanup:
     return NULL;
 }
 
-static struct mp_image *upload(struct vf_instance *vf, struct mp_image *in)
+static void vf_vavpp_process(struct mp_filter *f)
 {
-    struct vf_priv_s *p = vf->priv;
-    struct mp_image *out = mp_image_pool_get(p->pool, IMGFMT_VAAPI, in->w, in->h);
-    if (!out)
-        return NULL;
-    if (va_surface_upload(out, in) < 0) {
-        talloc_free(out);
-        return NULL;
-    }
-    mp_image_copy_attributes(out, in);
-    return out;
-}
+    struct priv *p = f->priv;
 
-static int filter_ext(struct vf_instance *vf, struct mp_image *in)
-{
-    struct vf_priv_s *p = vf->priv;
+    update_pipeline(f);
 
-    update_pipeline(vf);
+    mp_refqueue_execute_reinit(p->queue);
 
-    if (in && in->imgfmt != IMGFMT_VAAPI) {
-        struct mp_image *tmp = upload(vf, in);
-        talloc_free(in);
-        in = tmp;
-        if (!in)
-            return -1;
-    }
+    if (!mp_refqueue_can_output(p->queue))
+        return;
 
-    mp_refqueue_add_input(p->queue, in);
-    return 0;
-}
-
-static int filter_out(struct vf_instance *vf)
-{
-    struct vf_priv_s *p = vf->priv;
-
-    if (!mp_refqueue_has_output(p->queue))
-        return 0;
-
-    // no filtering
     if (!p->pipe.num_filters || !mp_refqueue_should_deint(p->queue)) {
+        // no filtering
         struct mp_image *in = mp_refqueue_get(p->queue, 0);
-        vf_add_output_frame(vf, mp_image_new_ref(in));
-        mp_refqueue_next(p->queue);
-        return 0;
+        mp_refqueue_write_out_pin(p->queue, mp_image_new_ref(in));
+    } else {
+        mp_refqueue_write_out_pin(p->queue, render(f));
     }
-
-    struct mp_image *out = render(vf);
-    mp_refqueue_next_field(p->queue);
-    if (!out)
-        return -1; // cannot render
-    vf_add_output_frame(vf, out);
-    return 0;
 }
 
-static int reconfig(struct vf_instance *vf, struct mp_image_params *in,
-                    struct mp_image_params *out)
+static void uninit(struct mp_filter *vf)
 {
-    struct vf_priv_s *p = vf->priv;
-
-    flush_frames(vf);
-    talloc_free(p->pool);
-    p->pool = NULL;
-
-    p->params = *in;
-
-    p->current_rt_format = VA_RT_FORMAT_YUV420;
-    p->pool = mp_image_pool_new(20);
-    va_pool_set_allocator(p->pool, p->va, p->current_rt_format);
-
-    struct mp_image *probe = mp_image_pool_get(p->pool, IMGFMT_VAAPI, in->w, in->h);
-    if (!probe)
-        return -1;
-    va_surface_init_subformat(probe);
-    *out = *in;
-    out->imgfmt = probe->params.imgfmt;
-    out->hw_subfmt = probe->params.hw_subfmt;
-    talloc_free(probe);
-
-    return 0;
-}
-
-static void uninit(struct vf_instance *vf)
-{
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = vf->priv;
     for (int i = 0; i < p->num_buffers; i++)
         vaDestroyBuffer(p->display, p->buffers[i]);
     if (p->context != VA_INVALID_ID)
         vaDestroyContext(p->display, p->context);
     if (p->config != VA_INVALID_ID)
         vaDestroyConfig(p->display, p->config);
-    talloc_free(p->pool);
+    av_buffer_unref(&p->hw_pool);
     flush_frames(vf);
-    mp_refqueue_free(p->queue);
+    talloc_free(p->queue);
+    av_buffer_unref(&p->av_device_ref);
 }
 
-static int query_format(struct vf_instance *vf, unsigned int imgfmt)
-{
-    struct vf_priv_s *p = vf->priv;
-    if (imgfmt == IMGFMT_VAAPI || va_image_format_from_imgfmt(p->va, imgfmt))
-        return vf_next_query_format(vf, IMGFMT_VAAPI);
-    return 0;
-}
-
-static int control(struct vf_instance *vf, int request, void* data)
-{
-    switch (request){
-    case VFCTRL_SEEK_RESET:
-        flush_frames(vf);
-        return true;
-    default:
-        return CONTROL_UNKNOWN;
-    }
-}
-
-static int va_query_filter_caps(struct vf_instance *vf, VAProcFilterType type,
+static int va_query_filter_caps(struct mp_filter *vf, VAProcFilterType type,
                                 void *caps, unsigned int count)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = vf->priv;
     VAStatus status = vaQueryVideoProcFilterCaps(p->display, p->context, type,
                                                  caps, &count);
-    return check_error(vf, status, "vaQueryVideoProcFilterCaps()") ? count : 0;
+    return CHECK_VA_STATUS(vf, "vaQueryVideoProcFilterCaps()") ? count : 0;
 }
 
-static VABufferID va_create_filter_buffer(struct vf_instance *vf, int bytes,
+static VABufferID va_create_filter_buffer(struct mp_filter *vf, int bytes,
                                           int num, void *data)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = vf->priv;
     VABufferID buffer;
     VAStatus status = vaCreateBuffer(p->display, p->context,
                                      VAProcFilterParameterBufferType,
                                      bytes, num, data, &buffer);
-    return check_error(vf, status, "vaCreateBuffer()") ? buffer : VA_INVALID_ID;
+    return CHECK_VA_STATUS(vf, "vaCreateBuffer()") ? buffer : VA_INVALID_ID;
 }
 
-static bool initialize(struct vf_instance *vf)
+static bool initialize(struct mp_filter *vf)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct priv *p = vf->priv;
     VAStatus status;
 
     VAConfigID config;
     status = vaCreateConfig(p->display, VAProfileNone, VAEntrypointVideoProc,
                             NULL, 0, &config);
-    if (!check_error(vf, status, "vaCreateConfig()")) // no entrypoint for video porc
+    if (!CHECK_VA_STATUS(vf, "vaCreateConfig()")) // no entrypoint for video porc
         return false;
     p->config = config;
 
     VAContextID context;
     status = vaCreateContext(p->display, p->config, 0, 0, 0, NULL, 0, &context);
-    if (!check_error(vf, status, "vaCreateContext()"))
+    if (!CHECK_VA_STATUS(vf, "vaCreateContext()"))
         return false;
     p->context = context;
 
     VAProcFilterType filters[VAProcFilterCount];
     int num_filters = VAProcFilterCount;
     status = vaQueryVideoProcFilters(p->display, p->context, filters, &num_filters);
-    if (!check_error(vf, status, "vaQueryVideoProcFilters()"))
+    if (!CHECK_VA_STATUS(vf, "vaQueryVideoProcFilters()"))
         return false;
 
     VABufferID buffers[VAProcFilterCount];
@@ -426,14 +377,28 @@ static bool initialize(struct vf_instance *vf)
         buffers[i] = VA_INVALID_ID;
     for (int i = 0; i < num_filters; i++) {
         if (filters[i] == VAProcFilterDeinterlacing) {
-            if (p->deint_type < 1)
-                continue;
             VAProcFilterCapDeinterlacing caps[VAProcDeinterlacingCount];
             int num = va_query_filter_caps(vf, VAProcFilterDeinterlacing, caps,
                                            VAProcDeinterlacingCount);
             if (!num)
                 continue;
-            VAProcDeinterlacingType algorithm = deint_algorithm[p->deint_type];
+            if (p->opts->deint_type < 0) {
+                for (int n = MP_ARRAY_SIZE(deint_algorithm) - 1; n > 0; n--) {
+                    for (int x = 0; x < num; x++) {
+                        if (caps[x].type == deint_algorithm[n]) {
+                            p->opts->deint_type = n;
+                            MP_VERBOSE(vf, "Selected deinterlacing algorithm: "
+                                       "%d\n", deint_algorithm[n]);
+                            goto found;
+                        }
+                    }
+                }
+                found: ;
+            }
+            if (p->opts->deint_type <= 0)
+                continue;
+            VAProcDeinterlacingType algorithm =
+                deint_algorithm[p->opts->deint_type];
             for (int n=0; n < num; n++) { // find the algorithm
                 if (caps[n].type != algorithm)
                     continue;
@@ -447,42 +412,69 @@ static bool initialize(struct vf_instance *vf)
                 MP_WARN(vf, "Selected deinterlacing algorithm not supported.\n");
         } // check other filters
     }
+    if (p->opts->deint_type < 0)
+        p->opts->deint_type = 0;
     p->num_buffers = 0;
     if (buffers[VAProcFilterDeinterlacing] != VA_INVALID_ID)
         p->buffers[p->num_buffers++] = buffers[VAProcFilterDeinterlacing];
-    p->do_deint = !!p->deint_type;
+    p->do_deint = !!p->opts->deint_type;
     // next filters: p->buffers[p->num_buffers++] = buffers[next_filter];
     return true;
 }
 
-static int vf_open(vf_instance_t *vf)
+static const struct mp_filter_info vf_vavpp_filter = {
+    .name = "vavpp",
+    .process = vf_vavpp_process,
+    .reset = flush_frames,
+    .destroy = uninit,
+    .priv_size = sizeof(struct priv),
+};
+
+static struct mp_filter *vf_vavpp_create(struct mp_filter *parent, void *options)
 {
-    struct vf_priv_s *p = vf->priv;
+    struct mp_filter *f = mp_filter_create(parent, &vf_vavpp_filter);
+    if (!f) {
+        talloc_free(options);
+        return NULL;
+    }
 
-    vf->reconfig = reconfig;
-    vf->filter_ext = filter_ext;
-    vf->filter_out = filter_out;
-    vf->query_format = query_format;
-    vf->uninit = uninit;
-    vf->control = control;
+    mp_filter_add_pin(f, MP_PIN_IN, "in");
+    mp_filter_add_pin(f, MP_PIN_OUT, "out");
 
-    p->queue = mp_refqueue_alloc();
+    struct priv *p = f->priv;
+    p->opts = talloc_steal(p, options);
+    p->config = VA_INVALID_ID;
+    p->context = VA_INVALID_ID;
 
-    p->va = hwdec_devices_load(vf->hwdec_devs, HWDEC_VAAPI);
-    if (!p->va)
-        return 0;
-    p->display = p->va->display;
-    if (initialize(vf))
-        return true;
-    uninit(vf);
-    return false;
+    p->queue = mp_refqueue_alloc(f);
+
+    p->av_device_ref = mp_filter_load_hwdec_device(f, AV_HWDEVICE_TYPE_VAAPI);
+    if (!p->av_device_ref)
+        goto error;
+
+    AVHWDeviceContext *hwctx = (void *)p->av_device_ref->data;
+    AVVAAPIDeviceContext *vactx = hwctx->hwctx;
+
+    p->display = vactx->display;
+
+    mp_refqueue_add_in_format(p->queue, IMGFMT_VAAPI, 0);
+
+    if (!initialize(f))
+        goto error;
+
+    return f;
+
+error:
+    talloc_free(f);
+    return NULL;
 }
 
-#define OPT_BASE_STRUCT struct vf_priv_s
+#define OPT_BASE_STRUCT struct opts
 static const m_option_t vf_opts_fields[] = {
     OPT_CHOICE("deint", deint_type, 0,
-               // The values must match with deint_algorithm[].
-               ({"no", 0},
+               // The values >=0 must match with deint_algorithm[].
+               ({"auto", -1},
+                {"no", 0},
                 {"first-field", 1},
                 {"bob", 2},
                 {"weave", 3},
@@ -493,11 +485,17 @@ static const m_option_t vf_opts_fields[] = {
     {0}
 };
 
-const vf_info_t vf_info_vaapi = {
-    .description = "VA-API Video Post-Process Filter",
-    .name = "vavpp",
-    .open = vf_open,
-    .priv_size = sizeof(struct vf_priv_s),
-    .priv_defaults = &vf_priv_default,
-    .options = vf_opts_fields,
+const struct mp_user_filter_entry vf_vavpp = {
+    .desc = {
+        .description = "VA-API Video Post-Process Filter",
+        .name = "vavpp",
+        .priv_size = sizeof(OPT_BASE_STRUCT),
+        .priv_defaults = &(const OPT_BASE_STRUCT){
+            .deint_type = -1,
+            .interlaced_only = 0,
+            .reversal_bug = 1,
+        },
+        .options = vf_opts_fields,
+    },
+    .create = vf_vavpp_create,
 };
