@@ -1,20 +1,18 @@
 /*
  * This file is part of mpv.
  *
- * mpv is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * mpv is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with mpv.  If not, see <http://www.gnu.org/licenses/>.
- *
- * Almost LGPL.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MPLAYER_MP_IMAGE_H
@@ -30,26 +28,16 @@
 #include "csputils.h"
 #include "video/img_format.h"
 
-#define MP_PALETTE_SIZE (256 * 4)
+// Assumed minimum align needed for image allocation. It's notable that FFmpeg's
+// libraries except libavcodec don't really know what alignment they want.
+// Things will randomly crash or get slower if the alignment is not satisfied.
+// Whatever. This value should be pretty safe with current CPU architectures.
+#define MP_IMAGE_BYTE_ALIGN 64
 
 #define MP_IMGFIELD_TOP_FIRST 0x02
 #define MP_IMGFIELD_REPEAT_FIRST 0x04
 #define MP_IMGFIELD_INTERLACED 0x20
 
-enum mp_spherical_type {
-    MP_SPHERICAL_AUTO = 0,
-    MP_SPHERICAL_NONE,              // normal video
-    MP_SPHERICAL_UNKNOWN,           // unknown projection
-    MP_SPHERICAL_EQUIRECTANGULAR,   // (untiled)
-};
-
-extern const struct m_opt_choice_alternatives mp_spherical_names[];
-
-struct mp_spherical_params {
-    enum mp_spherical_type type;
-    float ref_angles[3]; // yaw/pitch/roll, refer to AVSphericalMapping
-};
-
 // Describes image parameters that usually stay constant.
 // New fields can be added in the future. Code changing the parameters should
 // usually copy the whole struct, so that fields added later will be preserved.
@@ -62,9 +50,7 @@ struct mp_image_params {
     enum mp_chroma_location chroma_location;
     // The image should be rotated clockwise (0-359 degrees).
     int rotate;
-    enum mp_stereo3d_mode stereo_in;    // image is encoded with this mode
-    enum mp_stereo3d_mode stereo_out;   // should be displayed with this mode
-    struct mp_spherical_params spherical;
+    enum mp_stereo3d_mode stereo3d; // image is encoded with this mode
 };
 
 /* Memory management:
@@ -103,6 +89,8 @@ typedef struct mp_image {
     double pts;
     /* only after decoder */
     double dts, pkt_duration;
+    /* container reported FPS; can be incorrect, or 0 if unknown */
+    double nominal_fps;
     /* for private use */
     void* priv;
 
@@ -117,8 +105,18 @@ typedef struct mp_image {
     struct AVBufferRef *hwctx;
     // Embedded ICC profile, if any
     struct AVBufferRef *icc_profile;
+    // Closed captions packet, if any (only after decoder)
+    struct AVBufferRef *a53_cc;
+    // Other side data we don't care about.
+    struct mp_ff_side_data *ff_side_data;
+    int num_ff_side_data;
 } mp_image_t;
 
+struct mp_ff_side_data {
+    int type;
+    struct AVBufferRef *buf;
+};
+
 int mp_chroma_div_up(int size, int shift);
 
 int mp_image_get_alloc_size(int imgfmt, int w, int h, int stride_align);
@@ -129,7 +127,6 @@ struct mp_image *mp_image_from_buffer(int imgfmt, int w, int h, int stride_align
 
 struct mp_image *mp_image_alloc(int fmt, int w, int h);
 void mp_image_copy(struct mp_image *dmpi, struct mp_image *mpi);
-void mp_image_copy_gpu(struct mp_image *dst, struct mp_image *src);
 void mp_image_copy_attributes(struct mp_image *dmpi, struct mp_image *mpi);
 struct mp_image *mp_image_new_copy(struct mp_image *img);
 struct mp_image *mp_image_new_ref(struct mp_image *img);
@@ -151,6 +148,8 @@ void mp_image_setfmt(mp_image_t* mpi, int out_fmt);
 void mp_image_steal_data(struct mp_image *dst, struct mp_image *src);
 void mp_image_unref_data(struct mp_image *img);
 
+int mp_image_approx_byte_size(struct mp_image *img);
+
 struct mp_image *mp_image_new_dummy_ref(struct mp_image *img);
 struct mp_image *mp_image_new_custom_ref(struct mp_image *img, void *arg,
                                          void (*free)(void *arg));
@@ -185,6 +184,4 @@ void memcpy_pic(void *dst, const void *src, int bytesPerLine, int height,
 void memset_pic(void *dst, int fill, int bytesPerLine, int height, int stride);
 void memset16_pic(void *dst, int fill, int unitsPerLine, int height, int stride);
 
-void mp_check_gpu_memcpy(struct mp_log *log, bool *once);
-
 #endif /* MPLAYER_MP_IMAGE_H */
