@@ -317,6 +317,11 @@ class Window: NSWindow, NSWindowDelegate {
     }
 
     override func setFrame(_ frameRect: NSRect, display flag: Bool) {
+        if frameRect.width < minSize.width || frameRect.height < minSize.height {
+            mpv?.sendVerbose("tried to set too small window size: \(frameRect.size)")
+            return
+        }
+
         super.setFrame(frameRect, display: flag)
 
         if let size = unfsContentFrame?.size, keepAspect {
@@ -456,6 +461,12 @@ class Window: NSWindow, NSWindowDelegate {
         mpv?.command("set window-scale \(scale)")
     }
 
+    func addWindowScale(_ scale: Double) {
+        if !isInFullscreen {
+            mpv?.command("add window-scale \(scale)")
+        }
+    }
+
     func windowDidChangeScreen(_ notification: Notification) {
         if screen == nil {
             return
@@ -476,7 +487,7 @@ class Window: NSWindow, NSWindowDelegate {
 
     func windowDidChangeBackingProperties(_ notification: Notification) {
         cocoaCB.layer?.contentsScale = backingScaleFactor
-        cocoaCB.flagEvents(VO_EVENT_WIN_STATE)
+        cocoaCB.flagEvents(VO_EVENT_DPI)
     }
 
     func windowWillStartLiveResize(_ notification: Notification) {
