@@ -867,15 +867,6 @@ static int vo_cocoa_control_on_main_thread(struct vo *vo, int request, void *arg
     struct vo_cocoa_state *s = vo->cocoa;
 
     switch (request) {
-    case VOCTRL_FULLSCREEN:
-        return vo_cocoa_fullscreen(vo);
-    case VOCTRL_GET_FULLSCREEN:
-        *(int *)arg = s->fullscreen;
-        return VO_TRUE;
-    case VOCTRL_ONTOP:
-        return vo_cocoa_ontop(vo);
-    case VOCTRL_BORDER:
-        return vo_cocoa_window_border(vo);
     case VOCTRL_GET_UNFS_WINDOW_SIZE: {
         int *sz = arg;
         NSRect rect = (s->fullscreen || vo->opts->fullscreen) ?
@@ -894,11 +885,6 @@ static int vo_cocoa_control_on_main_thread(struct vo *vo, int request, void *arg
         queue_new_video_size(vo, r.size.width, r.size.height);
         return VO_TRUE;
     }
-    case VOCTRL_GET_WIN_STATE: {
-        const bool minimized = [[s->view window] isMiniaturized];
-        *(int *)arg = minimized ? VO_WIN_STATE_MINIMIZED : 0;
-        return VO_TRUE;
-    }
     case VOCTRL_SET_CURSOR_VISIBILITY:
         s->cursor_visibility_wanted = *(bool *)arg;
         return vo_cocoa_update_cursor_visibility(vo, false);
@@ -1048,7 +1034,6 @@ int vo_cocoa_control(struct vo *vo, int *events, int request, void *arg)
 {
     struct vo_cocoa_state *s = self.vout->cocoa;
     s->fullscreen = 1;
-    s->pending_events |= VO_EVENT_FULLSCREEN_STATE;
     vo_cocoa_anim_unlock(self.vout);
 }
 
@@ -1056,7 +1041,6 @@ int vo_cocoa_control(struct vo *vo, int *events, int request, void *arg)
 {
     struct vo_cocoa_state *s = self.vout->cocoa;
     s->fullscreen = 0;
-    s->pending_events |= VO_EVENT_FULLSCREEN_STATE;
     vo_cocoa_anim_unlock(self.vout);
 }
 
@@ -1109,16 +1093,6 @@ int vo_cocoa_control(struct vo *vo, int *events, int request, void *arg)
     vo_cocoa_update_cursor_visibility(self.vout, false);
 }
 
-- (void)windowDidMiniaturize:(NSNotification *)notification
-{
-    flag_events(self.vout, VO_EVENT_WIN_STATE);
-}
-
-- (void)windowDidDeminiaturize:(NSNotification *)notification
-{
-    flag_events(self.vout, VO_EVENT_WIN_STATE);
-}
-
 - (void)windowWillMove:(NSNotification *)notification
 {
     self.vout->cocoa->window_is_dragged = true;
