@@ -49,6 +49,8 @@ struct sws_opts {
     int chr_hshift;
     float chr_sharpen;
     float lum_sharpen;
+    int fast;
+    int bitexact;
     int zimg;
 };
 
@@ -73,27 +75,29 @@ const struct m_sub_options sws_conf = {
         OPT_INT("chs", chr_hshift, 0),
         OPT_FLOATRANGE("ls", lum_sharpen, 0, -100.0, 100.0),
         OPT_FLOATRANGE("cs", chr_sharpen, 0, -100.0, 100.0),
+        OPT_FLAG("fast", fast, 0),
+        OPT_FLAG("bitexact", bitexact, 0),
         OPT_FLAG("allow-zimg", zimg, 0),
         {0}
     },
     .size = sizeof(struct sws_opts),
     .defaults = &(const struct sws_opts){
-        .scaler = SWS_BICUBIC,
+        .scaler = SWS_LANCZOS,
     },
 };
 
 // Highest quality, but also slowest.
-const int mp_sws_hq_flags = SWS_LANCZOS | SWS_FULL_CHR_H_INT |
-                            SWS_FULL_CHR_H_INP | SWS_ACCURATE_RND |
-                            SWS_BITEXACT;
+static const int mp_sws_hq_flags = SWS_FULL_CHR_H_INT | SWS_FULL_CHR_H_INP |
+                                   SWS_ACCURATE_RND;
 
 // Fast, lossy.
 const int mp_sws_fast_flags = SWS_BILINEAR;
 
 // Set ctx parameters to global command line flags.
-void mp_sws_set_from_cmdline(struct mp_sws_context *ctx, struct mpv_global *g)
+static void mp_sws_update_from_cmdline(struct mp_sws_context *ctx)
 {
-    struct sws_opts *opts = mp_get_config_group(NULL, g, &sws_conf);
+    m_config_cache_update(ctx->opts_cache);
+    struct sws_opts *opts = ctx->opts_cache->opts;
 
     sws_freeFilter(ctx->src_filter);
     ctx->src_filter = sws_getDefaultFilter(opts->lum_gblur, opts->chr_gblur,
@@ -103,14 +107,12 @@ void mp_sws_set_from_cmdline(struct mp_sws_context *ctx, struct mpv_global *g)
 
     ctx->flags = SWS_PRINT_INFO;
     ctx->flags |= opts->scaler;
+    if (!opts->fast)
+        ctx->flags |= mp_sws_hq_flags;
+    if (opts->bitexact)
+        ctx->flags |= SWS_BITEXACT;
 
-    ctx->opts_allow_zimg = opts->zimg;
-
-    talloc_free(opts);
-
-#if HAVE_ZIMG
-    mp_zimg_set_from_cmdline(ctx->zimg, g);
-#endif
+    ctx->allow_zimg = opts->zimg;
 }
 
 bool mp_sws_supported_format(int imgfmt)
@@ -125,7 +127,7 @@ bool mp_sws_supports_formats(struct mp_sws_context *ctx,
                              int imgfmt_out, int imgfmt_in)
 {
 #if HAVE_ZIMG
-    if (ctx->allow_zimg && ctx->opts_allow_zimg) {
+    if (ctx->allow_zimg) {
         if (mp_zimg_supports_in_format(imgfmt_in) &&
             mp_zimg_supports_out_format(imgfmt_out))
             return true;
@@ -154,7 +156,8 @@ static bool cache_valid(struct mp_sws_context *ctx)
            ctx->brightness == old->brightness &&
            ctx->contrast == old->contrast &&
            ctx->saturation == old->saturation &&
-           ctx->allow_zimg == old->allow_zimg;
+           ctx->allow_zimg == old->allow_zimg &&
+           (!ctx->opts_cache || !m_config_cache_update(ctx->opts_cache));
 }
 
 static void free_mp_sws(void *p)
@@ -178,7 +181,6 @@ struct mp_sws_context *mp_sws_alloc(void *talloc_ctx)
         .force_reload = true,
         .params = {SWS_PARAM_DEFAULT, SWS_PARAM_DEFAULT},
         .cached = talloc_zero(ctx, struct mp_sws_context),
-        .opts_allow_zimg = true,
     };
     talloc_set_destructor(ctx, free_mp_sws);
 
@@ -190,6 +192,23 @@ struct mp_sws_context *mp_sws_alloc(void *talloc_ctx)
     return ctx;
 }
 
+// Enable auto-update of parameters from command line. Don't try to set custom
+// options (other than possibly .src/.dst), because they might be overwritten
+// if the user changes any options.
+void mp_sws_enable_cmdline_opts(struct mp_sws_context *ctx, struct mpv_global *g)
+{
+    if (ctx->opts_cache)
+        return;
+
+    ctx->opts_cache = m_config_cache_alloc(ctx, g, &sws_conf);
+    ctx->force_reload = true;
+    mp_sws_update_from_cmdline(ctx);
+
+#if HAVE_ZIMG
+    mp_zimg_enable_cmdline_opts(ctx->zimg, g);
+#endif
+}
+
 // Reinitialize (if needed) - return error code.
 // Optional, but possibly useful to avoid having to handle mp_sws_scale errors.
 int mp_sws_reinit(struct mp_sws_context *ctx)
@@ -204,12 +223,15 @@ int mp_sws_reinit(struct mp_sws_context *ctx)
     if (cache_valid(ctx))
         return 0;
 
+    if (ctx->opts_cache)
+        mp_sws_update_from_cmdline(ctx);
+
     sws_freeContext(ctx->sws);
     ctx->sws = NULL;
     ctx->zimg_ok = false;
 
 #if HAVE_ZIMG
-    if (ctx->allow_zimg && ctx->opts_allow_zimg) {
+    if (ctx->allow_zimg) {
         ctx->zimg->log = ctx->log;
         ctx->zimg->src = *src;
         ctx->zimg->dst = *dst;
@@ -229,11 +251,6 @@ int mp_sws_reinit(struct mp_sws_context *ctx)
     mp_image_params_guess_csp(src); // sanitize colorspace/colorlevels
     mp_image_params_guess_csp(dst);
 
-    struct mp_imgfmt_desc src_fmt = mp_imgfmt_get_desc(src->imgfmt);
-    struct mp_imgfmt_desc dst_fmt = mp_imgfmt_get_desc(dst->imgfmt);
-    if (!src_fmt.id || !dst_fmt.id)
-        return -1;
-
     enum AVPixelFormat s_fmt = imgfmt2pixfmt(src->imgfmt);
     if (s_fmt == AV_PIX_FMT_NONE || sws_isSupportedInput(s_fmt) < 1) {
         MP_ERR(ctx, "Input image format %s not supported by libswscale.\n",
@@ -254,12 +271,6 @@ int mp_sws_reinit(struct mp_sws_context *ctx)
     int d_csp = mp_csp_to_sws_colorspace(dst->color.space);
     int d_range = dst->color.levels == MP_CSP_LEVELS_PC;
 
-    // Work around libswscale bug #1852 (fixed in ffmpeg commit 8edf9b1fa):
-    // setting range flags for RGB gives random bogus results.
-    // Newer libswscale always ignores range flags for RGB.
-    s_range = s_range && (src_fmt.flags & MP_IMGFLAG_YUV);
-    d_range = d_range && (dst_fmt.flags & MP_IMGFLAG_YUV);
-
     av_opt_set_int(ctx->sws, "sws_flags", ctx->flags, 0);
 
     av_opt_set_int(ctx->sws, "srcw", src->w, 0);
@@ -343,7 +354,7 @@ int mp_image_sw_blur_scale(struct mp_image *dst, struct mp_image *src,
                            float gblur)
 {
     struct mp_sws_context *ctx = mp_sws_alloc(NULL);
-    ctx->flags = mp_sws_hq_flags;
+    ctx->flags = SWS_LANCZOS | mp_sws_hq_flags;
     ctx->src_filter = sws_getDefaultFilter(gblur, gblur, 0, 0, 0, 0, 0);
     ctx->force_reload = true;
     int res = mp_sws_scale(ctx, dst, src);
