@@ -1,6 +1,7 @@
 from waftools import inflector
 from waftools.checks.generic import *
 from waflib import Utils
+from distutils.version import StrictVersion
 import os
 
 __all__ = ["check_pthreads", "check_iconv", "check_lua",
@@ -85,7 +86,7 @@ def check_lua(ctx, dependency_identifier):
 
 def check_wl_protocols(ctx, dependency_identifier):
     def fn(ctx, dependency_identifier):
-        ret = check_pkg_config_datadir("wayland-protocols", ">= 1.14")
+        ret = check_pkg_config_datadir("wayland-protocols", ">= 1.15")
         ret = ret(ctx, dependency_identifier)
         if ret != None:
             ctx.env.WL_PROTO_DIR = ret.split()[0]
@@ -97,7 +98,7 @@ def check_cocoa(ctx, dependency_identifier):
         fragment         = load_fragment('cocoa.m'),
         compile_filename = 'test.m',
         framework_name   = ['Cocoa', 'IOKit', 'OpenGL', 'QuartzCore'],
-        includes         = ctx.srcnode.abspath(),
+        includes         = [ctx.srcnode.abspath()],
         linkflags        = '-fobjc-arc')
 
     res = fn(ctx, dependency_identifier)
@@ -113,10 +114,13 @@ def check_cocoa(ctx, dependency_identifier):
     return res
 
 def check_swift(ctx, dependency_identifier):
+    minVer = StrictVersion("3.0.2")
     if ctx.env.SWIFT_VERSION:
-        major = int(ctx.env.SWIFT_VERSION.split('.')[0])
-        ctx.add_optional_message(dependency_identifier,
-                                 'version found: ' + ctx.env.SWIFT_VERSION)
-        if major >= 3:
+        if StrictVersion(ctx.env.SWIFT_VERSION) >= minVer:
+            ctx.add_optional_message(dependency_identifier,
+                                     'version found: ' + str(ctx.env.SWIFT_VERSION))
             return True
+    ctx.add_optional_message(dependency_identifier,
+                             "'swift >= " + str(minVer) + "' not found, found " +
+                             str(ctx.env.SWIFT_VERSION or None))
     return False
