@@ -4,7 +4,8 @@
 
 #include "osdep/macosx_compat.h"
 
-int main(int argc, char **argv) {
+int main(int argc, char **argv)
+{
     @autoreleasepool {
         NSArray *ary = @[@1, @2, @3];
         NSLog(@"test subscripting: %@", ary[0]);
