@@ -174,7 +174,7 @@ def build(ctx):
             ( "osdep/macos/mpv_helper.swift" ),
             ( "osdep/macos/swift_extensions.swift" ),
             ( "osdep/macos/swift_compat.swift" ),
-            ( "osdep/macos/remote_command_center.swift", "macos-10-12-2-features" ),
+            ( "osdep/macos/remote_command_center.swift", "macos-media-player" ),
             ( "video/out/cocoa-cb/events_view.swift" ),
             ( "video/out/cocoa-cb/video_layer.swift" ),
             ( "video/out/cocoa-cb/window.swift" ),
@@ -410,6 +410,7 @@ def build(ctx):
 
         ## Video
         ( "video/csputils.c" ),
+        ( "video/cuda.c",                        "cuda-hwaccel" ),
         ( "video/d3d.c",                         "d3d-hwaccel" ),
         ( "video/decode/vd_lavc.c" ),
         ( "video/filter/refqueue.c" ),
@@ -460,9 +461,9 @@ def build(ctx):
         ( "video/out/gpu/utils.c" ),
         ( "video/out/gpu/video.c" ),
         ( "video/out/gpu/video_shaders.c" ),
-        ( "video/out/hwdec/hwdec_cuda.c",        "cuda-hwaccel" ),
-        ( "video/out/hwdec/hwdec_cuda_gl.c",     "cuda-hwaccel && gl" ),
-        ( "video/out/hwdec/hwdec_cuda_vk.c",     "cuda-hwaccel && vulkan" ),
+        ( "video/out/hwdec/hwdec_cuda.c",        "cuda-interop" ),
+        ( "video/out/hwdec/hwdec_cuda_gl.c",     "cuda-interop && gl" ),
+        ( "video/out/hwdec/hwdec_cuda_vk.c",     "cuda-interop && vulkan" ),
         ( "video/out/hwdec/hwdec_vaapi.c",       "vaapi-egl || vaapi-vulkan" ),
         ( "video/out/hwdec/hwdec_vaapi_gl.c",    "vaapi-egl" ),
         ( "video/out/hwdec/hwdec_vaapi_vk.c",    "vaapi-vulkan" ),
@@ -736,6 +737,9 @@ def build(ctx):
         if ctx.env.ZSHDIR:
             ctx.install_as(ctx.env.ZSHDIR + '/_mpv', 'etc/_mpv.zsh')
 
+        if ctx.env.BASHDIR:
+            ctx.install_as(ctx.env.BASHDIR + '/mpv', 'etc/mpv.bash-completion')
+
         ctx.install_files(
             ctx.env.DATADIR + '/applications',
             ['etc/mpv.desktop'] )
